@@ -1,4 +1,3 @@
-
 # -*- coding: ISO-8859-1 -*-
 
 # Copyright (C) 2003 Jrg Lehmann <joerg@luga.de>
@@ -18,9 +17,8 @@
 # along with PyTone; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from __future__ import nested_scopes
-import ConfigParser, curses, sys, getopt, exceptions, os.path, types, re
-import version
+import ConfigParser, copy, curses, sys, getopt, exceptions, os.path, types, re, types
+import log, version
 
 
 class ConfigError(exceptions.Exception):
@@ -39,6 +37,10 @@ class configsection:
             return self._configitems[name].get()
         except KeyError:
             raise AttributeError
+            try:
+                return self._configsections[name]
+            except KeyError:
+                raise AttributeError
 
     def __setattr__(self, name, value):
         self._configitems[name].set(value)
@@ -51,15 +53,19 @@ class configsection:
             return self._configitems[name].get()
         except KeyError:
             try:
-                return eval("self.%s" % name)
-            except:
-                raise AttributeError
+                return self._configsections[name]
+            except KeyError:
+                raise IndexError
+
     def asdict(self):
         d = {}
         for n, v in self._configitems.items():
             d[n] = v
         return d
 
+    def getsubsections(self):
+        return self._configsections.keys()
+
 #
 # define different types of configuration variables
 #
@@ -121,6 +127,16 @@ class configfloat(configitem):
         return float(s)
 
 
+class configlist(configitem):
+    def _check(self, s):
+        try:
+            items = s.split()
+        except:
+            raise ConfigError("Expecting list, got '%s'" % s)
+
+    def _convert(self, s):
+        return s.split()
+
 
 class configcolor(configitem):
 
@@ -280,11 +296,14 @@ class configpath(configitem):
     def _convert(self, s):
         return os.path.expanduser(s)
 
+
 class configre(configitem):
     def _convert(self, s):
         return re.compile(s)
 
 
+
+
 BORDER_TOP = 1
 BORDER_BOTTOM = 2
 BORDER_LEFT = 4
@@ -294,7 +313,7 @@ BORDER_ULTRACOMPACT = 32
 
 class configborder(configitem):
     def _check(self, s):
-        if s == "all" or s == "compact" or s == "off" or s == "ultracompact": 
+        if s == "all" or s == "compact" or s == "off" or s == "ultracompact":
             return
         for b in s.split():
             if b not in ("top", "bottom", "left", "right", "compact"):
@@ -336,10 +355,10 @@ class general(configsection):
     layout = configalternatives("twocolumn", ["onecolumn", "twocolumn"])
     throttleoutput = configint("0")
     autoplaymode = configalternatives("off", ["off", "repeat", "random"])
+    plugins = configlist("")
 
 class database(configsection):
-
-    class main(configsection):
+    class __template__(configsection):
         type = configalternatives("local", ["local", "remote"])
         basename = configpath("~/.pytone/mp3")
         dbenvdir = configpath("~/.pytone/mp3")
@@ -355,23 +374,6 @@ class database(configsection):
         server = configstring("")
         port = configint("1972")
 
-    class secondary(configsection):
-        type = configalternatives("off", ["local", "remote", "off"])
-        basename = configpath("~/.pytone/mp3_2")
-        dbenvdir = configpath("~/.pytone/mp3_2")
-        dbfile = configpath("")
-        cachesize = configint("1000")
-        musicbasedir = configpath("")
-        tracknrandtitlere = configre(r"^\[?(\d+)\]? ?[- ] ?(.*)\.(mp3|ogg)$")
-        tags_capitalize = configboolean("true")
-        tags_stripleadingarticle = configboolean("true")
-        tags_removeaccents = configboolean("true")
-        autoregisterer = configboolean("on")
-        playingstatslength = configint("100")
-        server = configstring("")
-        port = configint("1972")
-
-
 class mixer(configsection):
     device = configpath("/dev/mixer")
     channel = configstring("SOUND_MIXER_PCM")
@@ -433,6 +435,7 @@ class filelistwindow(configsection):
     scrollbar = configboolean("true")
     scrollmode = configalternatives("page", ["page", "line"])
     virtualdirectoriesattop = configboolean("false")
+    skipsinglealbums = configboolean("true")
 
 
 class playerwindow(configsection):
@@ -582,6 +585,8 @@ class keybindings(configsection):
         playerpause = configkeys("p P")
         playernextsong = configkeys("n N")
         playerprevioussong = configkeys("b B")
+        playerforward = configkeys(">")
+        playerrewind = configkeys("<")
         playerstop = configkeys("S")
         playlistdeleteplayedsongs = configkeys("KEY_BACKSPACE")
         playlistclear = configkeys("ctrl-d")
@@ -597,14 +602,14 @@ class keybindings(configsection):
         volumedown = configkeys("(")
 
     class filelistwindow(configsection):
-        selectnext = configkeys("KEY_DOWN")
-        selectprev = configkeys("KEY_UP")
+        selectnext = configkeys("KEY_DOWN j")
+        selectprev = configkeys("KEY_UP k")
         selectnextpage = configkeys("ctrl-n KEY_NPAGE")
         selectprevpage = configkeys("ctrl-p KEY_PPAGE")
         selectfirst = configkeys("ctrl-a KEY_HOME")
         selectlast = configkeys("ctrl-e KEY_END")
-        dirdown = configkeys("KEY_RIGHT KEY_SPACE \n KEY_ENTER")
-        dirup = configkeys("KEY_LEFT")
+        dirdown = configkeys("KEY_RIGHT KEY_SPACE \n KEY_ENTER l")
+        dirup = configkeys("KEY_LEFT h")
         addsongtoplaylist = configkeys("KEY_SPACE \n KEY_ENTER KEY_RIGHT")
         adddirtoplaylist = configkeys("i I KEY_IC alt-KEY_RIGHT")
         playselectedsong = configkeys("alt-\n alt-KEY_ENTER")
@@ -615,8 +620,8 @@ class keybindings(configsection):
         repeatsearch = configkeys("ctrl-g")
 
     class playlistwindow(configsection):
-        selectnext = configkeys("KEY_DOWN")
-        selectprev = configkeys("KEY_UP")
+        selectnext = configkeys("KEY_DOWN j")
+        selectprev = configkeys("KEY_UP k")
         selectnextpage = configkeys("ctrl-n KEY_NPAGE")
         selectprevpage = configkeys("ctrl-p KEY_PPAGE")
         selectfirst = configkeys("ctrl-a KEY_HOME")
@@ -624,10 +629,11 @@ class keybindings(configsection):
         moveitemup = configkeys("+")
         moveitemdown = configkeys("-")
         deleteitem = configkeys("d D KEY_DC")
-        activatefilelist = configkeys("\t KEY_LEFT KEY_RIGHT")
+        activatefilelist = configkeys("\t KEY_LEFT h")
         playselectedsong = configkeys("alt-\n alt-KEY_ENTER")
         rescan = configkeys("u U")
         shuffle = configkeys("r R")
+        filelistjumptoselectedsong = configkeys("KEY_RIGHT l")
 
 #
 # register known configuration sections
@@ -641,62 +647,100 @@ sections = ['mixerwindow', 'helpwindow', 'filelistwindow', 'database', 'iteminfo
 # end configuration tree
 ##############################################################################
 
+# options which can be overwritten via the command line
+
+userconfigfile = os.path.expanduser("~/.pytone/pytonerc")
+forcedatabaserebuild = False
+forcedebugfile = None
+
 #
 # helper functions
 #
 
-# has containing command line arguments convert to config options
-cmdlineargs = {}
+# configparser used for the config files
+configparser = None
 
-# read configuration from config files
-def readconfig():
-    cflist = ["/etc/pytonerc"]
-    if cmdlineargs.has_key("configfile"):
-        cflist.append(cmdlineargs["configfile"])
-        del cmdlineargs["configfile"]
-    else:
-         cflist.append(os.path.expanduser("~/.pytone/pytonerc"))
-    cp = ConfigParser.RawConfigParser()
-    cp.read(cflist)
+def setupconfigparser():
+    """ initialize ConfigParser.RawConfigParser for the standard configuration files """
+    global configparser
+    cflist = ["/etc/pytonerc", userconfigfile]
+    s = ", ".join(map(os.path.realpath, cflist))
+    log.info("Using configuration from file(s) %s" % s)
+    configparser = ConfigParser.RawConfigParser()
+    configparser.read(cflist)
 
-    for section in cp.sections():
-        if "." not in section:
-            # not a subsection
-            if section not in sections:
-                raise ConfigError("Unkown configuration section '%s'" % section)
-        else:
-            # check for valid subsection
-            try:
-                mainsection, subsection = section.split(".")
-            except:
-                raise ConfigError("Unkown configuration section '%s'" % section)
-            if mainsection not in sections or not subsection.isalpha():
-                raise ConfigError("Unkown configuration section '%s'" % section)
-            if not eval("issubclass(%s, configsection)" % section):
-                raise ConfigError("Unkown configuration section '%s'" % section)
 
-        clsection = eval(section)
-        for option in cp.options(section):
+def readconfigsection(section, clsection):
+    """ fill configsection subclass clsection with entries stored under the name section in configfile and
+    return a corresponding clsection instance"""
+    try:
+        for option in configparser.options(section):
             if not clsection.__dict__.has_key(option):
                 raise ConfigError("Unkown configuration option '%s' in section '%s'" %
                                   (option, section))
-            value = cp.get(section, option)
+            value = configparser.get(section, option)
             clsection.__dict__[option].set(value)
+    except ConfigParser.NoSectionError:
+        pass
+
+
+def finishconfigsection(clsection):
+    """ finish a configsection subclass clsection """
+    # move configuration items from class __dict__ into _configitems and store
+    # config subsections
+    clsection._configitems = {}
+    clsection._configsections = {}
+    for n, v in clsection.__dict__.items():
+        if isinstance(v, configitem):
+            clsection._configitems[n] = v
+            del clsection.__dict__[n]
+        elif type(v) == types.ClassType and issubclass(v, configsection) and n != "__template__":
+            finishconfigsection(v)
+            # instantiate class to make __getattr__ and __setattr__ work and add the instance to
+            # a _configsections dictionary
+            clsection.__dict__[n] = clsection._configsections[n] = v()
+
+
+def processstandardconfig():
+    for section in configparser.sections():
+        if not section.startswith("plugin."):
+            if "." not in section:
+                # not a subsection
+                if section not in sections:
+                    raise ConfigError("Unkown configuration section '%s'" % section)
+            else:
+                # check for valid subsection
+                # first check the form (section.subsection) of the config section
+                try:
+                    mainsection, subsection = section.split(".")
+                except:
+                    raise ConfigError("Unkown configuration section '%s'" % section)
+                if mainsection not in sections or not subsection.isalpha():
+                    raise ConfigError("Unkown configuration section '%s'" % section)
+                # get corresponding config class
+                try:
+                    clsection = eval(section)
+                    if not issubclass(clsection, configsection):
+                        raise ConfigError("Unkown configuration section '%s'" % section)
+                except AttributeError:
+                    # if it does not exist, use a __template__ class in
+                    # mainsection if possible
+                    try:
+                        # We copy the __template__ class by explicitely
+                        # creating a new class with a deeply copied class
+                        # dictionary (maybe there is an easier solution)
+                        # Any
+                        templateclassdict = eval("%s.__template__.__dict__" % mainsection)
+                        newclass = types.ClassType(mainsection+subsection, (configsection,), copy.deepcopy(templateclassdict))
+                        exec("%s.%s = newclass" % (mainsection, subsection))
+                    except AttributeError:
+                        raise ConfigError("Unkown configuration section '%s'" % section)
+
+            readconfigsection(section, eval(section))
 
-# prepare all configuration sections for later use
 
 def finishconfig():
-    def finishconfigsection(clsection):
-        # move configuration items for class __dict__ into _configitems
-        clsection._configitems = {}
-        for n, v in clsection.__dict__.items():
-            if isinstance(v, configitem):
-                clsection._configitems[n] = v
-                del clsection.__dict__[n]
-            elif type(v)==types.ClassType and issubclass(v, configsection):
-                finishconfigsection(v)
-                # instantiate class to make __getattr__ and __setattr__ work
-                clsection.__dict__[n] = v()
+    """ prepare all configuration sections for later use and read command line arguments """
 
     for section in sections:
         finishconfigsection(eval(section))
@@ -705,9 +749,13 @@ def finishconfig():
         # to make __getattr__ and __setattr__ work
         exec("%s = %s()" % (section, section)) in globals(), globals()
 
-    # set command line arguments
-    for key, value in cmdlineargs.items():
-        exec("%s=%s" % (key, value)) in globals(), globals()
+    # apply command line options
+    if forcedatabaserebuild:
+        for databasename in database.getsubsections():
+            exec("database.%s.autoregisterer = 'on'" % databasename) in globals(), globals()
+
+    if forcedebugfile:
+        general.debugfile = forcedebugfile
 
 
 def gendefault():
@@ -740,7 +788,7 @@ def gendefault():
 
 def usage():
     print "PyTone %s" % version.version
-    print "Copyright (C) 2002, 2003 Jrg Lehmann <joerg@luga.de>"
+    print "Copyright %s" % version.copyright
     print "usage: pytone.py [options]"
     print "-h, --help: show this help"
     print "-c, --config <filename>: read config from filename"
@@ -749,6 +797,11 @@ def usage():
 
 
 def processcommandline():
+    # we pass the information on the command line options via
+    # global variables
+    global userconfigfile
+    global forcedatabaserebuild
+    global forcedebugfile
     try:
         # keep rest of arguments for other use
         global args
@@ -764,12 +817,11 @@ def processcommandline():
             usage()
             sys.exit()
         if o in ("-d", "--debug"):
-            cmdlineargs["general.debugfile"] = '"%s"' % a
+            forcedebugfile = a
         if o in ("-c", "--config"):
-            cmdlineargs["configfile"] = a
+            userconfigfile = a
         if o in ("-r", "--rebuild"):
-            cmdlineargs["database.main.autoregisterer"] = '"on"'
-            cmdlineargs["database.secondary.autoregisterer"] = '"on"'
+            forcedatabaserebuild = True
 
 
 def checkoptions():
@@ -778,23 +830,39 @@ def checkoptions():
         print "Error: cannot run both as server and as client"
         sys.exit(2)
 
-    if database.main.type=="local" and database.main.musicbasedir=="":
-        print "Please set musicbasedir in the [database.main] section of the config file pytonerc\nto the location of your MP3/Ogg Vorbis files."
+
+    # check database options
+    basenames = []
+    dbenvdirs = []
+
+    if not database.getsubsections():
+        print "Please define at least one song database in your configuration file."
         sys.exit(2)
 
-    if database.main.type=="local" and not (database.main.dbfile!="" and database.main.basename=="" or
-                                            database.main.dbfile=="" and database.main.basename!=""):
-        print "Please use either the dbfile or the basename option (not both) to specify the location of your song database."
-        sys.exit(2)
+    for databasename in database.getsubsections():
+        songdb = database[databasename]
 
-    if database.secondary.type=="local" and database.secondary.musicbasedir=="":
-        print "Please set musicbasedir in the [database.secondary] section of the config file pytonerc\nto the location of your MP3/Ogg Vorbis files."
-        sys.exit(2)
+        if songdb == "local" and songdb.musicbasedir == "":
+            print ( "Please set musicbasedir in the [database.%s] section of the config file pytonerc\n"
+                    "to the location of your MP3/Ogg Vorbis files." % databasename )
+            sys.exit(2)
 
-    if database.secondary.type=="local" and not (database.secondary.dbfile!="" and database.secondary.basename=="" or
-                                                 database.secondary.dbfile=="" and database.secondary.basename!=""):
-        print "Please use either the dbfile or the basename option (not both) to specify the location of your song database."
-        sys.exit(2)
+        if songdb.type=="local" and not (songdb.dbfile != "" and songdb.basename == "" or
+                                         songdb.dbfile == "" and songdb.basename != ""):
+            print "Please use either the dbfile or the basename option (not both) to specify the location of your song database '%s'." % databasename
+            sys.exit(2)
+
+        if songdb.basename != "":
+            if songdb.basename in basenames:
+                print "basename '%s' of database '%s' already in use." % (songdb.basename, databasename)
+                sys.exit(2)
+            basenames.append(songdb.basename)
+
+        if songdb.dbfile != "":
+            if songdb.dbenvdir in dbenvdirs:
+                print "dbenvdir '%s' of database '%s' already in use." % (songdb.dbenvdir, databasename)
+                sys.exit(2)
+            dbenvdirs.append(songdb.dbenvdir)
 
     # check whether oss module is present
     try:
@@ -804,14 +872,16 @@ def checkoptions():
             import oss
         except:
             mixer.device = ""
-            # log.debug("disabling mixer since neither ossaudiodev nor oss module is installed")
+            log.warning("disabling mixer since neither ossaudiodev nor oss module is installed")
 
+    # check ao options
     for aooption in player.main.aooptions.split() + player.secondary.aooptions.split():
         if aooption.count("=")!=1:
             raise RuntimeError("invalid format for alsa option '%s'" % aooption)
 
 
 def processconfig():
-    readconfig()
+    setupconfigparser()
+    processstandardconfig()
     finishconfig()
     checkoptions()
