@@ -1,8 +1,6 @@
 # -*- coding: ISO-8859-1 -*-
 
-# Copyright (C) 2002, 2003 Jrg Lehmann <joerg@luga.de>
-#
-# Ogg Vorbis interface by Byron Ellacott <bje@apnic.net>.
+# Copyright (C) 2002, 2003, 2004, 2005 Jrg Lehmann <joerg@luga.de>
 #
 # This file is part of PyTone (http://www.luga.de/pytone/)
 #
@@ -20,22 +18,12 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os.path, locale, re, string, sys, time
-import log
-
-try:
-    import eyeD3
-    eyeD3present = 1
-    log.info("using eyeD3 module for id3 tag parsing")
-except:
-    log.info("using integrated MP3Info module for id3 tag parsing")
-    eyeD3present = 0
-    
-import MP3Info
+import log, metadata
 
 fallbacklocalecharset = 'iso-8859-1'
 
 # Try to determine "correct" character set for the reencoding of the
-# unicode strings contained in Ogg Vorbis files 
+# unicode strings contained in Ogg Vorbis files
 try:
     # works only in python > 2.3
     localecharset = locale.getpreferredencoding()
@@ -50,13 +38,6 @@ except:
 if localecharset in [None, 'ANSI_X3.4-1968']:
     localecharset = fallbacklocalecharset
 
-try:
-    import ogg.vorbis
-    oggsupport = 1
-except:
-    oggsupport = 0
-    log.info("Ogg Vorbis support disabled, since module is not present")
-    pass
 
 tracknrandtitlere = re.compile("^\[?(\d+)\]? ?[- ] ?(.*)\.(mp3|ogg)$")
 UNKNOWN = "Unknown"
@@ -70,13 +51,16 @@ class dbitem:
 
     def __cmp__(self, other):
         try:
-            return cmp(self.__class__, other.__class__) or cmp(self.id, other.id)
+            return cmp(self.id, other.id)
         except:
             return 1
 
     def __repr__(self):
         return "%s(%s)" % (self.__class__, self.id)
 
+    def __hash__(self):
+        return hash(self.id)
+
 
 class song(dbitem):
 
@@ -86,13 +70,11 @@ class song(dbitem):
         self.name = os.path.basename(self.id)
 
         # we set the path later in readid3info
-        self.path = None 
+        self.path = None
 
-        if self.id.lower().endswith(".mp3"):
-            self.type = "mp3"
-        elif self.id.lower().endswith(".ogg"):
-            self.type = "ogg"
-        else:
+        # determine type of file from its extension
+        self.type = metadata.gettype(os.path.splitext(relpath)[1])
+        if self.type is None:
             raise RuntimeError("Fileformat of song '%s' not supported" % (self.id))
 
         # song metadata
@@ -103,7 +85,7 @@ class song(dbitem):
         self.genre = ""
         self.tracknr = ""
         self.length = 0
-        
+
         # statistical information
         self.nrplayed = 0
         self.lastplayed = []
@@ -127,91 +109,6 @@ class song(dbitem):
         else:
             raise AttributeError
 
-    def readid3tag_eyeD3(self):
-        mp3file = eyeD3.Mp3AudioFile(self.path)
-        mp3info = mp3file.getTag()
-
-        # we definitely want the length of the MP3 file, even if no ID3 tag is present,
-        # so extract this info before anything goes wrong
-        self.length = mp3file.getPlayTime()
-
-        self.title = mp3info.getTitle()
-        self.title = self.title.encode(localecharset, 'replace')
-        self.title = MP3Info._strip_zero(self.title)
-
-        self.album = mp3info.getAlbum()
-        self.album = self.album.encode(localecharset, 'replace')
-        self.album = MP3Info._strip_zero(self.album)
-
-        self.artist = mp3info.getArtist()
-        self.artist = self.artist.encode(localecharset, 'replace')
-        self.artist = MP3Info._strip_zero(self.artist)
-
-        self.year = mp3info.getYear()
-        if self.year:
-            self.year = self.year.encode(localecharset, 'replace')
-
-        try:
-            self.genre = mp3info.getGenre()
-            if self.genre:
-                self.genre = self.genre.getName()
-        except eyeD3.tag.GenreException, e:
-            self.genre = e.msg.split(':')[1].strip()
-
-        self.tracknr = str(mp3info.getTrackNum()[0])
-
-        # if the playtime is also in the ID3 tag information, we
-        # try to read it from there
-        if mp3info.frames["TLEN"]:
-            length = None
-            try:
-                length = int(int(mp3info.frames["TLEN"])/1000)
-            except:
-                # time in seconds (?), possibly with bad decimal separator, e.g "186,333"
-                try:
-                    length = int(float(mp3info.frames["TLEN"].replace(",", ".")))
-                except:
-                    pass
-            if length:
-                self.length = length
-
-    def readid3tag_MP3Info(self):
-        mp3file = open(self.path, "rb")
-        mp3info = MP3Info.MP3Info(mp3file)
-        self.title = mp3info.title
-        self.album = mp3info.album
-        self.artist = mp3info.artist
-        self.year = mp3info.year
-        self.genre  = mp3info.genre
-        self.tracknr = mp3info.track
-        if mp3info.frames.tags.has_key("TLEN"):
-            try:
-                self.length = int(mp3info.id3.tags["TLEN"])/1000
-            except:
-                # time in seconds (?), possibly with bad decimal separator, e.g "186,333"
-                t = mp3info.id3.tags["TLEN"].replace(",", ".")
-                self.length = int(float(t))
-        else:
-            self.length = mp3file.getPlayTime()
-        mp3file.close()
-
-    def readid3tag_Vorbis(self):
-        vf = ogg.vorbis.VorbisFile(self.path)
-        id3get = vf.comment().as_dict().get
-        self.title = id3get('TITLE', [""])[0]
-        self.title = self.title.encode(localecharset, 'replace')
-        self.album = id3get('ALBUM', [""])[0]
-        self.album = self.album.encode(localecharset, 'replace')
-        self.artist = id3get('ARTIST', [""])[0]
-        self.artist = self.artist.encode(localecharset, 'replace')
-        self.year = id3get('DATE', [""])[0]
-        self.year = self.year.encode(localecharset, 'replace')
-        self.genre  = id3get('GENRE', [""])[0]
-        self.genre = self.genre.encode(localecharset, 'replace')
-        self.tracknr = id3get('TRACKNUMBER', [""])[0]
-        self.tracknr = self.tracknr.encode(localecharset, 'replace')
-        self.length = vf.time_total(0)
-
     def scanfile(self, basedir, tracknrandtitlere, capitalize, stripleadingarticle, removeaccents):
         """ update path info for song and scan id3 information """
         self.path = os.path.normpath(os.path.join(basedir, self.id))
@@ -239,17 +136,26 @@ class song(dbitem):
         fntitle = fntitle.replace("_", " ")
         fnalbum = fnalbum.replace("_", " ")
         fnartist = fnartist.replace("_", " ")
-                
+
         try:
-            if self.type=="mp3":
-                if eyeD3present:
-                    self.readid3tag_eyeD3()
-                else:
-                    self.readid3tag_MP3Info()
-            else:
-                self.readid3tag_Vorbis()
+            metadatadecoder = metadata.getmetadatadecoder(self.type)
         except:
-            pass
+            raise RuntimeError("Support for %s songs not enabled" % (self.type))
+
+        try:
+            log.debug("reading metadata for %s" % self.path)
+            md = metadatadecoder(self.path)
+            self.title = md.title
+            self.album = md.album
+            self.artist = md.artist
+            self.year = md.year
+            self.genre = md.genre
+            self.tracknr = md.tracknr
+            self.length = md.length
+            log.debug("metadata for %s read successfully" % self.path)
+        except:
+            log.warning("could not read metadata for %s" % self.path)
+            log.debug_traceback()
 
         # sanity check for tracknr
         try:
@@ -269,7 +175,7 @@ class song(dbitem):
             self.title = fntitle
 
         # also try to use tracknr from filename, if not present as id3 tag
-        if not self.tracknr or self.tracknr=="0":
+        if not self.tracknr or self.tracknr == "0":
             self.tracknr = fntracknr
 
         # we don't want empty album names
@@ -356,7 +262,6 @@ class album(dbitem):
 
 class playlist(dbitem):
     def __init__(self, path):
-        assert os.path.isfile(path), "'%s' doesn't exist" % path
         self.path = self.id = os.path.normpath(path)
         self.name = os.path.basename(path)
         if self.name.endswith(".m3u"):
@@ -376,16 +281,18 @@ class playlist(dbitem):
                     self.songs.append(path)
         file.close()
 
+
 class dbindex(dbitem):
 
     """ base class for indices (besides albums and artists) in database """
-    
+
     def __init__(self, id):
         self.id = id
         self.artists = []
         self.albums = []
         self.songs = []
 
+
 class genre(dbindex):
     def __init__(self, name):
         dbindex.__init__(self, name)
