@@ -21,6 +21,18 @@ class event:
     def __str__(self):
         return self.__class__.__name__
 
+
+class dbevent(event):
+
+    """ base class for all database service events """
+
+    def __init__(self, songdbid):
+        self.songdbid = songdbid
+
+    def __str__(self):
+        return "%s(%s)" % (self.__class__.__name__, self.songdbid)
+
+
 class quit(event):
     """ request end of thread """
     pass
@@ -48,7 +60,7 @@ class selectionchanged(event):
         self.item = item
 
     def __str__(self):
-        return "%s(%s)" % (self.__class__.__name__, `self.item`)
+        return "%s(%s)" % (self.__class__.__name__, self.item)
 
 
 class focuschanged(event):
@@ -75,7 +87,7 @@ class sendeventat(event):
         self.replace = replace
 
     def __str__(self):
-        return "%s(%s, %s, %s)" % (self.__class__.__name__, `self.event`, `self.alarmtime`, `self.repeat`, `self.replace`)
+        return "%s(%s, %s, %s)" % (self.__class__.__name__, self.event, self.alarmtime, self.repeat, self.replace)
 
 
 class sendeventin(event):
@@ -90,117 +102,129 @@ class sendeventin(event):
         self.replace = replace
 
     def __str__(self):
-        return "%s(%s, %s, %s, %s)" % (self.__class__.__name__, `self.event`, `self.alarmtimediff`, `self.repeat`, `self.replace`)
+        return "%s(%s, %s, %s, %s)" % (self.__class__.__name__, self.event, self.alarmtimediff, self.repeat, self.replace)
 
 
-class checkpointdatabase(event):
+class checkpointdb(dbevent):
     """flush memory pool, write checkpoint record to log and flush flog of songdbid"""
-    
-    def __init__(self, songdbid):
-        self.songdbid = songdbid
-
-    def __str__(self):
-        return "%s(%s)" % (self.__class__.__name__, `self.songdbid`)
 
 
-class updatesong(event):
+class updatesong(dbevent):
     """ update song in database """
-    
+
     def __init__(self, songdbid, song):
         self.songdbid = songdbid
         self.song = song
 
     def __str__(self):
-        return "%s(%s)->%s" % (self.__class__.__name__, `self.song`, `self.songdbid`)
+        return "%s(%s)->%s" % (self.__class__.__name__, self.song, self.songdbid)
 
 
-class rescansong(event):
+class rescansong(dbevent):
     """ reread id3 information of song (or delete it if it does not longer exist) """
-    
+
     def __init__(self, songdbid, song):
         self.songdbid = songdbid
         self.song = song
 
     def __str__(self):
-        return "%s(%s)->%s" % (self.__class__.__name__, `self.song`, `self.songdbid`)
+        return "%s(%s)->%s" % (self.__class__.__name__, self.song, self.songdbid)
 
 
-class delsong(event):
-    """ update song in database """
-    
+class delsong(dbevent):
+    """ delete song from database """
+
     def __init__(self, songdbid, song):
         self.songdbid = songdbid
         self.song = song
 
     def __str__(self):
-        return "%s(%s)->%s" % (self.__class__.__name__, `self.song`, `self.songdbid`)
+        return "%s(%s)->%s" % (self.__class__.__name__, self.song, self.songdbid)
 
-    
-class updatealbum(event):
+
+class updatealbum(dbevent):
     """ update album in database """
-    
+
     def __init__(self, songdbid, album):
         self.songdbid = songdbid
         self.album = album
 
     def __str__(self):
-        return "%s(%s)->%s" % (self.__class__.__name__, `self.album`, `self.songdbid`)
+        return "%s(%s)->%s" % (self.__class__.__name__, self.album, self.songdbid)
 
 
-class updateartist(event):
+class updateartist(dbevent):
     """ update artist in database """
-    
+
     def __init__(self, songdbid, artist):
         self.songdbid = songdbid
         self.artist = artist
 
     def __str__(self):
-        return "%s(%s)->%s" % (self.__class__.__name__, `self.artist`, `self.songdbid`)
+        return "%s(%s)->%s" % (self.__class__.__name__, self.artist, self.songdbid)
 
 
-class registersongs(event):
+class updateplaylist(dbevent):
+    """ update playlist in database """
+
+    def __init__(self, songdbid, playlist):
+        self.songdbid = songdbid
+        self.playlist = playlist
+
+    def __str__(self):
+        return "%s(%s)->%s" % (self.__class__.__name__, self.playlist, self.songdbid)
+
+
+class delplaylist(dbevent):
+    """ delete playlist from database """
+
+    def __init__(self, songdbid, playlist):
+        self.songdbid = songdbid
+        self.playlist = playlist
+
+    def __str__(self):
+        return "%s(%s)->%s" % (self.__class__.__name__, self.playlist, self.songdbid)
+
+
+class registersongs(dbevent):
     def __init__(self, songdbid, songs):
         self.songdbid = songdbid
         self.songs = songs
-        
+
     def __str__(self):
-        return "%s(%s)->%s" % (self.__class__.__name__, `self.songs`, `self.songdbid`)
+        return "%s(%s)->%s" % (self.__class__.__name__, self.songs, self.songdbid)
 
 
-class registerplaylists(event):
-    def __init__(self, songdbid, paths):
+class registerplaylists(dbevent):
+    def __init__(self, songdbid, playlists):
         self.songdbid = songdbid
-        self.paths = paths
-        
+        self.playlists = playlists
+
     def __str__(self):
-        return "%s(%s)->%s" % (self.__class__.__name__, self.paths, `self.songdbid`)
+        return "%s(%s)->%s" % (self.__class__.__name__, self.playlists, self.songdbid)
 
 
-class autoregistersongs(event):
-    def __init__(self, songdbid):
-        self.songdbid = songdbid
-        
-    def __str__(self):
-        return "%s->%s" % (self.__class__.__name__, `self.songdbid`)
+class autoregistersongs(dbevent):
+    """ start autoregisterer for database """
 
 
-class rescansongs(event):
+class rescansongs(dbevent):
     def __init__(self, songdbid, songs):
         self.songdbid = songdbid
         self.songs = songs
-        
+
     def __str__(self):
-        return "%s(%s)->%s" % (self.__class__.__name__, `self.songs`, `self.songdbid`)
+        return "%s(%s)->%s" % (self.__class__.__name__, self.songs, self.songdbid)
 
 
-class clearstats(event):
+class clearstats(dbevent):
     """ clear playing and added information of all songs (be carefull!) """
-    
+
     def __init__(self, songdbid):
         self.songdbid = songdbid
 
     def __str__(self):
-        return "%s->%s" % (self.__class__.__name__, `self.songdbid`)
+        return "%s->%s" % (self.__class__.__name__, self.songdbid)
 
 
 class songchanged(event):
@@ -208,62 +232,61 @@ class songchanged(event):
     def __init__(self, songdbid, song):
         self.songdbid = songdbid
         self.song = song
-        
+
     def __str__(self):
-        return "%s(%s)->%s" % (self.__class__.__name__, `self.song`, `self.songdbid`)
+        return "%s(%s)->%s" % (self.__class__.__name__, self.song, self.songdbid)
 
 
-class artistaddedtosongdb(event):
+class artistaddedordeleted(event):
     def __init__(self, songdbid, artist):
         self.songdbid = songdbid
         self.artist = artist
 
     def __str__(self):
-        return "%s(%s)->%s" % (self.__class__.__name__, `self.artist`, `self.songdbid`)
+        return "%s(%s)->%s" % (self.__class__.__name__, self.artist, self.songdbid)
 
-        
-class albumaddedtosongdb(event):
+
+class albumaddedordeleted(event):
     def __init__(self, songdbid, album):
         self.songdbid = songdbid
         self.album = album
 
     def __str__(self):
-        return "%s(%s)->%s" % (self.__class__.__name__, `self.album`, `self.songdbid`)
-    
+        return "%s(%s)->%s" % (self.__class__.__name__, self.album, self.songdbid)
+
+
+class dbplaylistchanged(event):
+    def __init__(self, songdbid, playlist):
+        self.songdbid = songdbid
+        self.playlist = playlist
+
+    def __str__(self):
+        return "%s(%s)->%s" % (self.__class__.__name__, self.playlist, self.songdbid)
+
 
 class requestnextsong(event):
     def __init__(self, playerid, previous=0):
         self.playerid = playerid
         self.previous = previous
-        
-    def __str__(self):
-        return "%s(%s, %s)" % (self.__class__.__name__, `self.playerid`, `self.previous`)
 
-
-class nextsong(event):
-    """ play song on player """
-    
-    def __init__(self, playerid, song):
-        self.playerid = playerid
-        self.song = song
-        
     def __str__(self):
-        return "%s(%s->%s)" % (self.__class__.__name__, `self.song`, `self.playerid`)
+        return "%s(%s, %s)" % (self.__class__.__name__, self.playerid, self.previous)
+
 
 class playerevent(event):
     """ event for the player control """
 
     def __init__(self, playerid):
         self.playerid = playerid
-        
+
     def __str__(self):
-        return "%s(%s)" % (self.__class__.__name__, `self.playerid`)
+        return "%s(%s)" % (self.__class__.__name__, self.playerid)
 
 
 class playerstart(playerevent):
     """ start player """
     pass
-    
+
 
 class playerpause(playerevent):
     """ pause player """
@@ -275,8 +298,8 @@ class playertogglepause(playerevent):
     pass
 
 
-class playerforward(playerevent):
-    """ forward player to next song """
+class playernext(playerevent):
+    """ play next song on player """
     pass
 
 
@@ -285,27 +308,49 @@ class playerprevious(playerevent):
     pass
 
 
+class playerseekrelative(playerevent):
+    """ seek relative in song by the given number of seconds """
+    
+    def __init__(self, playerid, seconds):
+        self.playerid = playerid
+        self.seconds = seconds
+
+    def __str__(self):
+        return "%s(%f->%s)" % (self.__class__.__name__, self.seconds, self.playerid)
+
+
 class playerstop(playerevent):
     """ stop player """
     pass
 
 
+class playerplaysong(playerevent):
+    """ play song on player """
+
+    def __init__(self, playerid, song):
+        self.playerid = playerid
+        self.song = song
+
+    def __str__(self):
+        return "%s(%s->%s)" % (self.__class__.__name__, self.song, self.playerid)
+
+
 class playerratecurrentsong(playerevent):
     """ rate song currently being played """
     def __init__(self, playerid, rating):
         playerevent.__init__(self, playerid)
         self.rating = rating
-        
+
     def __str__(self):
-        return "%s(%s,%d)" % (self.__class__.__name__, `self.playerid`, self.rating)
+        return "%s(%s,%d)" % (self.__class__.__name__, self.playerid, self.rating)
 
 
 class playbackinfochanged(event):
     def __init__(self, playbackinfo):
         self.playbackinfo = playbackinfo
-        
+
     def __str__(self):
-        return "%s(%s)" % (self.__class__.__name__, `self.playbackinfo`)
+        return "%s(%s)" % (self.__class__.__name__, self.playbackinfo)
 
 
 class updatestatusbar(event):
@@ -315,7 +360,7 @@ class updatestatusbar(event):
     pos = 1: player info
     pos = 2: global info
     """
-    
+
     def __init__(self, pos, content):
         self.pos = pos
         self.content = content
@@ -332,7 +377,7 @@ class requestinput(event):
 
     def __str__(self):
         return "%s(%s,%s,%s)" % (self.__class__.__name__,
-                              self.title, self.prompt, `self.handler`)
+                              self.title, self.prompt, self.handler)
 
 
 class playlistevent(event):
@@ -343,26 +388,26 @@ class playlistaddsongs(playlistevent):
     """ add songs to playlist """
     def __init__(self, songs):
         self.songs = songs
-        
+
     def __str__(self):
-        return "%s(%s)" % (self.__class__.__name__, `self.songs`)
+        return "%s(%s)" % (self.__class__.__name__, self.songs)
 
 
 class playlistaddsongtop(playlistevent):
     """ add song to top of playlist """
     def __init__(self, song):
         self.song = song
-        
+
     def __str__(self):
-        return "%s(%s)" % (self.__class__.__name__, `self.song`)
+        return "%s(%s)" % (self.__class__.__name__, self.song)
 
 
 class playlistdeletesong(playlistevent):
     def __init__(self, id):
         self.id = id
-        
+
     def __str__(self):
-        return "%s(%s)" % (self.__class__.__name__, `self.id`)
+        return "%s(%s)" % (self.__class__.__name__, self.id)
 
 
 class playlistmovesongup(playlistevent):
@@ -410,25 +455,39 @@ class playlisttoggleautoplaymode(playlistevent):
     pass
 
 
+class playlistplaysong(playlistevent):
+    """ immediately play song in playlist """
+    def __init__(self, id):
+        self.id = id
+
+    def __str__(self):
+        return "%s(%s)" % (self.__class__.__name__, self.id)
+
+
 class playlistchanged(event):
-    def __init__(self, songs, ptime, ttime, autoplaymode, playingsong):
+    def __init__(self, songs, ptime, ttime, autoplaymode, playingitem):
         self.songs = songs
         self.ptime = ptime
         self.ttime = ttime
         self.autoplaymode = autoplaymode
-        self.playingsong = playingsong
-        
+        self.playingitem = playingitem
+
     def __str__(self):
         return "%s(%s,%s/%s,%s,%s)" % (self.__class__.__name__,
-                                       `self.songs`, `self.ptime`, `self.ttime`, `self.autoplaymode`,
-                                       `self.playingsong`)
+                                       self.songs, self.ptime, self.ttime, self.autoplaymode,
+                                       self.playingitem)
 
+class filelistjumptosong(event):
+    """ jump to specific song in filelist window """
+    def __init__(self, song):
+        self.song = song
+
+    def __str__(self):
+        return "%s(%s)" % (self.__class__.__name__, self.song)
 
 class hidewindow(event):
     def __init__(self, window):
         self.window = window
-        
+
     def __str__(self):
-        return "%s(%s)" % (self.__class__.__name__, `self.window`)
-
-
+        return "%s(%s)" % (self.__class__.__name__, self.window)
