@@ -25,7 +25,7 @@ import slist
 class filelist(slist.slist):
 
     def __init__(self, win, songdbids):
-        slist.slist.__init__(self, win, config.filelistwindow.scrollmode=="page")
+        slist.slist.__init__(self, win, config.filelistwindow.scrollmode == "page")
 
         self.basedir = item.basedir(songdbids, win.ih,
                                     config.filelistwindow.virtualdirectoriesattop)
@@ -33,33 +33,36 @@ class filelist(slist.slist):
         self.shistory = []
         self.readdir()
 
-        self.win.channel.subscribe(events.artistaddedtosongdb,
-                                   self.artistaddedtosongdb)
-        self.win.channel.subscribe(events.albumaddedtosongdb,
-                                   self.albumaddedtosongdb)
-
-    def set(self, items):
-        slist.slist.set(self, items)
+        self.win.channel.subscribe(events.artistaddedordeleted, self.artistaddedordeleted)
+        self.win.channel.subscribe(events.albumaddedordeleted, self.albumaddedordeleted)
+        self.win.channel.subscribe(events.dbplaylistchanged, self.dbplaylistchanged)
+        self.win.channel.subscribe(events.filelistjumptosong, self.filelistjumptosong)
 
     def isdirselected(self):
         return isinstance(self.getselected(), item.diritem)
 
     def issongselected(self):
         return isinstance(self.getselected(), item.song)
-        
+
     def getselectedsubdir(self):
         return self.dir + [self.getselected()]
 
     def readdir(self):
         self.set(self.dir[-1].getcontents())
-        return 1
+
+    def updatedir(self):
+        """ reread directory trying to keep the current selection """
+        self.set(self.dir[-1].getcontents(), keepselection=True)
 
     def dirdown(self):
         self.shistory.append((self.dir, self.selected, self.top))
         self.dir = self.getselectedsubdir()
-        
-        if self.readdir() is None:
-            self.dir, self.selected, self.top = self.shistory.pop()
+        self.readdir()
+        # In the case of the selected item having been an artist check
+        # whether only one album is present. If yes directly jump to
+        # this album.
+        if config.filelistwindow.skipsinglealbums and self.dir[-1].isartist() and len(self) <= 2:
+            self.dir = self.getselectedsubdir()
             self.readdir()
 
     def dirup(self):
@@ -71,26 +74,14 @@ class filelist(slist.slist):
             self.top = top
             self._notifyselectionchanged()
 
-    def jumptorandomlist(self):
-        if self.dir != [self.basedir, self.basedir.randomlist]:
-            self.shistory.append((self.dir, self.selected, self.top))
-        self.dir = [self.basedir, self.basedir.randomlist]
-        self.readdir()
-
-    def jumptofilesystembase(self):
-        if self.dir != [self.basedir, self.basedir.filesystembase]:
-            self.shistory.append((self.dir, self.selected, self.top))
-        self.dir = [self.basedir, self.basedir.filesystembase]
-        self.readdir()
-        
     def selectionpath(self):
         return self.dir[-1].getheader(self.getselected())
-        
+
     def insertrecursiveselection(self):
         if self.isdirselected():
             songs = self.getselected().getcontentsrecursivesorted()
             hub.notify(events.playlistaddsongs(songs))
-        elif elf.issongselected():
+        elif self.issongselected():
             hub.notify(events.playlistaddsongs([self.getselected()]))
 
     def randominsertrecursiveselection(self):
@@ -101,7 +92,6 @@ class filelist(slist.slist):
             hub.notify(events.playlistaddsongs([self.getselected()]))
 
     def rateselection(self, rating):
-        assert 0<=rating<=5, "only ratings between 0 and 5 are allowed"
         if (isinstance(self.getselected(), item.song) or
             isinstance(self.getselected(), item.album) or
             isinstance(self.getselected(), item.artist)):
@@ -121,23 +111,48 @@ class filelist(slist.slist):
                 for song in self.getselected().getcontentsrecursive():
                     dsongs.setdefault(song.songdbid, []).append(song.song)
                 for songdbid, songs in dsongs.items():
-                    hub.notify(events.rescansongs(songdbid, songs))
+                    if songs:
+                        hub.notify(events.rescansongs(songdbid, songs))
         else:
             self.getselected().rescan()
 
     # event handler
 
-    def artistaddedtosongdb(self, event):
-        if isinstance(self.dir[-1], item.basedir) and event.songdbid==self.dir[-1].songdbid:
-            newartist = item.artist(event.songdbid, event.artist.name)
-            self.insertitem(newartist, self.dir[-1].cmpitem)
-            self.win.update()
+    def artistaddedordeleted(self, event):
+        #if isinstance(self.dir[-1], item.basedir):
+        self.updatedir()
+        self.win.update()
 
-    def albumaddedtosongdb(self, event):
-        if (isinstance(self.dir[-1], item.artist) and
-            self.dir[-1].songdbid==event.songdbid and
-            self.dir[-1].name in event.album.artists):
-            newalbum = item.album(event.songdbid,
-                                  event.album.id, self.dir[-1].name, event.album.name)
-            self.insertitem(newalbum, self.dir[-1].cmpitem)
-            self.win.update()
+    def albumaddedordeleted(self, event):
+        #if (isinstance(self.dir[-1], item.artist) and
+        #    self.dir[-1].songdbid==event.songdbid and
+        #    self.dir[-1].name in event.album.artists):
+        self.updatedir()
+        self.win.update()
+
+    def dbplaylistchanged(self, event):
+        #if (isinstance(self.dir[-1], item.artist) and
+        #    self.dir[-1].songdbid==event.songdbid and
+        #    self.dir[-1].name in event.album.artists):
+        self.updatedir()
+        self.win.update()
+
+    def filelistjumptosong(self, event):
+        """ directly jump to given song """
+        # In order to get the correct shistory, we more or less simulate
+        # a walk through the directory hierarchy, starting from the basedir.
+        self.shistory = []
+        self.dir = [self.basedir]
+        self.readdir()
+        if self.selectbyname(event.song.artist):
+            self.dirdown()
+        else:
+            return
+        # We might have skipped the album when there is only a single one of
+        # the given artist.
+        if not self.dir[-1].isalbum():
+            if self.selectbyname(event.song.album):
+                self.dirdown()
+            else:
+                return
+        self.selectbyname(event.song.name)
