@@ -42,6 +42,8 @@ descriptions = {
           "playerpause":       (_("pause"), _("pause main player")),
           "playernextsong":    (_("next song"), _("advance to next song")),
           "playerprevioussong":(_("previous song"), _("go back to previous song")),
+          "playerrewind":      (_("rewind"), _("rewind main player")),
+          "playerforward":     (_("forward"), _("forward main player")),
           "playerstop":        (_("stop"), _("stop main player")),
           "playlistdeleteplayedsongs":
                                (_("delete played"), _("delete played songs from playlist")),
@@ -91,6 +93,8 @@ descriptions = {
           "playselectedsong":  (_("immediate play"), _("play selected song immediately")),
           "activatefilelist":  (_("switch to database"), _("switch to database window")),
           "rescan":            (_("rescan"), _("rescan/update id3 info for selection")),
+          "filelistjumptoselectedsong":
+                               (_("jump to selected"), _("jump to selected song in filelist window")),
           }
     }
 
