@@ -145,15 +145,21 @@ class channel:
 	self.queue = PriorityQueue(-1)
 	# self.queue = Queue.Queue(-1)
 
-    def process(self, blocking=0):
+    def process(self, block=False, timeout=None):
 	""" process queued events and request
 
-	If blocking is set, we wait for incoming events and requests
+	If block is set, we wait for incoming events and requests.  In
+        this case, a timeout in seconds can be specified, as well.
 	"""
 
-	while not self.queue.empty() or blocking:
-	    item = self.queue.get()
-	    blocking = 0
+        while True:
+            try:
+                item = self.queue.get(block=block, timeout=timeout)
+            except Queue.Empty:
+                break
+            # after having get the first event, we do no longer block
+            block = False
+            timeout = None
 	    if isinstance(item, events.event):
 		try:
 		    for subscribedevent, handler in self.subscriptions:
