@@ -19,6 +19,7 @@
 
 import config
 import item
+import services.playlist
 import events, hub
 import window
 import messagewin
@@ -48,7 +49,7 @@ class iteminfowin(window.window):
                 self.settitle(_("Ogg Info"))
         else:
             self.settitle(_("Directory Info"))
-        
+
         window.window.update(self)
 
         # get lines to display
@@ -58,7 +59,7 @@ class iteminfowin(window.window):
         else:
             info = []
         l = info + empty*(4-len(info))
-        
+
         colsep = self.iw>45
 
         # calculate width of columns
@@ -86,10 +87,13 @@ class iteminfowin(window.window):
                 self.addstr(l[3][-1].ljust(wc4)[:wc4], self.colors.content)
 
     # event handler
-        
+
     def selectionchanged(self, event):
-        if self.player and isinstance(event.item, item.song) and self.item!=event.item:
-            hub.notify(events.nextsong(self.player, event.item))
+        if self.player and self.item!=event.item:
+            if isinstance(event.item, item.song):
+                hub.notify(events.playerplaysong(self.player, event.item))
+            elif isinstance(event.item, services.playlist.playlistitem):
+                hub.notify(events.playerplaysong(self.player, event.item.song))
         self.item = event.item
         self.update()
 
@@ -103,7 +107,7 @@ class iteminfowin(window.window):
 
 
 class iteminfowinlong(messagewin.messagewin):
-    
+
     def __init__(self, screen, maxh, maxw, channel):
         messagewin.messagewin.__init__(self, screen, maxh, maxw, channel,
                                        config.colors.iteminfolongwindow,
@@ -115,7 +119,7 @@ class iteminfowinlong(messagewin.messagewin):
         channel.subscribe(events.selectionchanged, self.selectionchanged)
 
     def _outputlen(self, width):
-        return 13
+        return 15
 
     def showitems(self):
         # get lines to display
@@ -125,7 +129,7 @@ class iteminfowinlong(messagewin.messagewin):
         else:
             info = []
         l = info + empty*(4-len(info))
-        
+
         colsep = self.iw>45
 
         # calculate width of columns
@@ -136,7 +140,7 @@ class iteminfowinlong(messagewin.messagewin):
             wc3 = max(wc3, len(line[2]))
         wc1 += colsep
         wc3 += colsep
-        wc4 = 10
+        wc4 = 0
         wc2 = self.iw-wc1-wc3-wc4-1
         self.clear()
         for lno in range(len(info)):
