@@ -18,8 +18,9 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import inspect
-import config
+import sys
 import time
+import traceback
 
 _DEBUG   = 0
 _INFO    = 1
@@ -38,14 +39,12 @@ LOGLEVEL = _INFO
 
 # open file for log output, if necessary
 debugfile = None
-try:
-    if config.general.debugfile:
-        debugfile = open(config.general.debugfile, "w", 1)
-except TypeError:
-    # If the config module was not initialized, we might get a type error here.
-    # In this case, we simply ignore the error and do not enable the log file.
-    pass
 
+def initdebugfile(debugfilename):
+    """ direct debugging output to debugfilename """
+    global debugfile
+    if debugfilename:
+        debugfile = open(debugfilename, "w", 1)
 
 # log buffer consisting of tuples (loglevel, time, logmessage)
 items = []
@@ -81,4 +80,8 @@ def warning(s):
 def error(s):
     log(s, _ERROR)
 
-info(_("PyTone startup"))
+def debug_traceback():
+    debug("Exception caught: %s " % sys.exc_info()[1])
+    tblist = traceback.extract_tb(sys.exc_info()[2])
+    for s in traceback.format_list(tblist):
+        debug(s[:-1])
