@@ -36,7 +36,7 @@ import config
 
 
 class mainscreen:
-    def __init__(self, screen, songdbids, playerids):
+    def __init__(self, screen, songdbids, playerids, plugins):
         self.screen = screen
         self.h, self.w = screen.getmaxyx()
         self.channel = hub.newchannel()
@@ -83,6 +83,11 @@ class mainscreen:
             del config.keybindings.general.volumeup
             del config.keybindings.general.volumedown
 
+        # now we start the plugins
+        for pluginmodule, pluginconfig in plugins:
+            plugin = pluginmodule.plugin(self.channel, pluginconfig, self)
+            plugin.start()
+
         self.channel.subscribe(events.keypressed, self.keypressed)
         self.channel.subscribe(events.activateplaylist, self.activateplaylist)
         self.channel.subscribe(events.activatefilelist, self.activatefilelist)
@@ -132,7 +137,7 @@ class mainscreen:
 
     def quit(self, event):
         """ cleanup """
-        self.done = 1
+        self.done = True
 
     def getmaxyx(self):
         # taken from http://groups.google.com/groups?selm=uk7xtwasm.fsf%40python.net
