@@ -107,7 +107,7 @@ class mixerwin(window.window):
     def keypressed(self, event):
         key = event.key
         if key in self.keybindings["volumeup"]:
-            self.changevolume(+self.stepsize)
+            self.changevolume(self.stepsize)
         elif key in self.keybindings["volumedown"]:
             self.changevolume(-self.stepsize)
         else:
