@@ -84,13 +84,13 @@ class servernetworkreceiver(threading.Thread):
 
     def _receiveobject(self):
 	line = self.rfile.readline()
+        if not line: return None, None
 	try:
             log.debug("server: request type received")
 	    type, bytes = line.split()
 	    bytes = int(bytes)
-	    if type!=_SENDFILE:
- 
-		objstring = self.rfile.read(bytes+2)[:-2]
+	    if type != _SENDFILE:
+ 		objstring = self.rfile.read(bytes+2)[:-2]
                 log.debug("server: object received")
                 obj = loads(objstring)
                 log.debug("server receive: type=%s object=%s" % (type, `obj`))
@@ -163,7 +163,7 @@ class handler(SocketServer.StreamRequestHandler, SocketServer.BaseRequestHandler
         # Process events coming from the rest of the PyTone server.
         # This sends (via eventhandler) subscribed events to the client
 	while not self.done:
-	    self.channel.process(blocking=True)
+	    self.channel.process(block=True)
 	log.debug("terminating handler")
 	self.channel.hub.disconnect(self.channel)
 
