@@ -125,12 +125,16 @@ class playerwin(window.window):
         elif key in self.keybindings["playerstart"]:
             hub.notify(events.playerstart(self.playerid))
         elif key in self.keybindings["playernextsong"]:
-            hub.notify(events.playerforward(self.playerid))
+            hub.notify(events.playernext(self.playerid))
         elif key in self.keybindings["playerprevioussong"]:
-            hub.notify(events.playerprevious(self.playerid))   
+            hub.notify(events.playerprevious(self.playerid))
+        elif key in self.keybindings["playerrewind"]:
+            hub.notify(events.playerseekrelative(self.playerid, -2))
+        elif key in self.keybindings["playerforward"]:
+            hub.notify(events.playerseekrelative(self.playerid, 2))
         elif key in self.keybindings["playerstop"]:
             hub.notify(events.playerstop(self.playerid))
-        elif ord("1")<=key-1024<=ord("5") and self.song:
+        elif ord("1") <= key-1024 <= ord("5") and self.song:
             self.song.rate(key-1024-ord("1")+1)
         else:
             return
