@@ -24,22 +24,24 @@ import slist
 #
 # playlist class, which acts as glue layer between the playlist service
 # and the playlist window
-# 
+#
 
 class playlist(slist.slist):
     def __init__(self, win, playerid):
         slist.slist.__init__(self, win, config.playlistwindow.scrollmode=="page")
         self.playerid = playerid
         self.songdbid = "main"
-        self.items, self.ptime, self.ttime, self.autoplaymode, self.playingsong = \
-                    hub.request(requests.playlistgetcontents())
-        
+        items, self.ptime, self.ttime, self.autoplaymode, self.playingitem = \
+               hub.request(requests.playlistgetcontents())
+        self.set(items)
+        self._recenter()
+
         self.win.channel.subscribe(events.playlistchanged, self.playlistchanged)
-        
+
     def _recenter(self):
         """ recenter playlist around currently playing (or alternatively last) song """
         for i in range(len(self)):
-            if self[i] is self.playingsong or i==len(self)-1:
+            if self[i] is self.playingitem or i==len(self)-1:
                 oldselected = self.selected
                 self.selected = i
                 if self.selected != oldselected:
@@ -54,77 +56,62 @@ class playlist(slist.slist):
                 self._updatetop()
                 break
 
-    def getselected(self):
-        playlistitem = slist.slist.getselected(self)
+    def getselectedsong(self):
+        """ return song corresponding to currently selected item or None """
+        playlistitem = self.getselected()
         if playlistitem:
             return playlistitem.song
         else:
             return None
 
+    # The following three slist.slist methods are delegated to the
+    # playlist service. Any resulting changes to the playlist will be
+    # performed only later when a playlistchanged event comes back
+
     def deleteselected(self):
         "delete currently selected item"
         if self.selected is not None:
-            hub.notify(events.playlistdeletesong(self.items[self.selected].id))
+            hub.notify(events.playlistdeletesong(self.getselected().id))
 
     def moveitemup(self):
         "move selected item up, if not first"
         if self.selected is not None and self.selected>0:
-            hub.notify(events.playlistmovesongup(self.items[self.selected].id))
+            hub.notify(events.playlistmovesongup(self.getselected().id))
 
     def moveitemdown(self):
         "move selected item down, if not last"
         if self.selected is not None and self.selected<len(self)-1:
-            hub.notify(events.playlistmovesongdown(self.items[self.selected].id))
+            hub.notify(events.playlistmovesongdown(self.getselected().id))
 
     def rateselection(self, rating):
-        assert 0<=rating<=5, "only ratings between 0 and 5 are allowed"
         if self.selected is not None:
-            self.getselected().rate(rating)
+            self.getselectedsong().rate(rating)
 
     def rescanselection(self):
         if self.selected is not None:
-            self.getselected().rescan()
+            self.getselectedsong().rescan()
 
     def playselected(self):
-        item = slist.slist.getselected(self)
+        item = self.getselected()
         if item:
-            # move item up before first not yet played song
-            if not item.hasbeenplayed():
-                self.deleteselected()
-            hub.notify(events.playlistaddsongtop(item.song))
+            hub.notify(events.playlistplaysong(item.id))
+
+    def filelistjumptoselected(self):
+        song = self.getselectedsong()
+        if song is not None:
+            hub.notify(events.filelistjumptosong(song))
 
     # event handler
 
     def playlistchanged(self, event):
-        oldlen = len(self)
-        if self.selected is not None:
-            oldselecteditem = slist.slist.getselected(self)
-            oldselected = self.selected
-            self.set(event.songs)
-            # we first try to select the previously selected song
-            try:
-                self.selected = self.index(oldselecteditem)
-            except ValueError:
-                # if this fails (typically because the song has been
-                # deleted from the playlist), we take the item at the
-                # last selected position, if possible
-                if oldselected<len(self):
-                    self.selected = oldselected
-                elif len(self)>0:
-                    self.selected = len(self)-1
-            self._notifyselectionchanged()
-            self._updatetop()
-        else:
-            self.set(event.songs)
-            
+        self.set(event.songs, keepselection=True)
         self.ptime = event.ptime
         self.ttime = event.ttime
         self.autoplaymode = event.autoplaymode
-        self.playingsong = event.playingsong
+        self.playingitem = event.playingitem
 
         # recenter window if the playlist window doesn't have the focus.
         if not self.win.hasfocus():
             self._recenter()
 
         self.win.update()
-
