@@ -38,9 +38,15 @@ class playlistwin(window.window):
                                config.colors.playlistwindow,
                                _("Playlist"),
                                border, config.playlistwindow.scrollbar)
-        
+
+        # Immediately remove focus from playlist window in order to
+        # prevent wrong selectionchanged events being issued, which
+        # would lead to playing the first song in the playlist on the
+        # second player (if configured)
+        self.bottom()
+
         self.playlist = playlist.playlist(self, playerid)
-        
+
         self.channel.subscribe(events.keypressed, self.keypressed)
         self.channel.subscribe(events.mouseevent, self.mouseevent)
         self.channel.subscribe(events.focuschanged, self.focuschanged)
@@ -103,6 +109,10 @@ class playlistwin(window.window):
                 self.playlist.rescanselection()
             elif ord("0")<=key<=ord("5"):
                 self.playlist.rateselection(key-ord("1")+1)
+            elif key in self.keybindings["filelistjumptoselectedsong"]:
+                self.playlist.filelistjumptoselected()
+                self.playlist._recenter()
+                hub.notify(events.activatefilelist())
             else:
                 return
 
@@ -121,7 +131,7 @@ class playlistwin(window.window):
                     if y==self.iy+1:
                         self.playlist.selectprev()
                     elif y==self.iy+self.ih-2:
-                        self.playlist.selectnext()                    
+                        self.playlist.selectnext()
                     elif self.iy<y<scrollbarbegin:
                         self.playlist.selectprevpage()
                     elif scrollbarbegin+scrollbarheight<=y<self.iy+self.ih-2:
@@ -166,17 +176,17 @@ class playlistwin(window.window):
             attr = curses.A_NORMAL
 
             if i<len(self.playlist):
-                song = self.playlist[i]
-                adddict = {"playstarthours":   time.localtime(song.playstarttime)[3],
-                           "playstartminutes": time.localtime(song.playstarttime)[4],
-                           "playstartseconds": time.localtime(song.playstarttime)[5]}
-                name = song.format(self.songformat, adddict=adddict)
-                if self.playlist.playingsong and song is self.playlist.playingsong:
+                item = self.playlist[i]
+                adddict = {"playstarthours":   time.localtime(item.playstarttime)[3],
+                           "playstartminutes": time.localtime(item.playstarttime)[4],
+                           "playstartseconds": time.localtime(item.playstarttime)[5]}
+                name = item.song.format(self.songformat, adddict=adddict)
+                if self.playlist.playingitem and item is self.playlist.playingitem:
                     if self.playlist.selected==i and self.hasfocus():
                         attr = self.colors.selected_playingsong
                     else:
                         attr = self.colors.playingsong
-                elif song.hasbeenplayed():
+                elif item.hasbeenplayed():
                     if i==self.playlist.selected and self.hasfocus():
                         attr = self.colors.selected_playedsong
                     else:
@@ -197,4 +207,3 @@ class playlistwin(window.window):
         # position/selection
 	if self.hasfocus() and self.playlist.selected is not None:
 	    self.win.move(self.playlist.selected-self.playlist.top+1, 1)
-
