@@ -24,10 +24,10 @@ import curses, os, os.path, signal, sys
 # gettext initialization.
 ##############################################################################
 
-# We have to do this very early, before importing our modules. We
-# assume that the locales lie in same dir as this module. This may not
-# be the case, if the .mo files are installed at their proper
-# location.
+# We have to initialize gettext very early, before importing our
+# modules. Assume that the locales lie in same dir as this
+# module. This may not be the case, if the .mo files are installed at
+# their proper location.
 
 try:
     import gettext
@@ -56,15 +56,25 @@ import config
 config.processcommandline()
 config.processconfig()
 
+# now that the configuration has been read, we can imort the log
+# module and initialize the debug file if necessary
+import log
+log.initdebugfile(config.general.debugfile)
+
+# log version (but do this after the debug file has been initialized,
+# such that the version gets included there)
+import version
+log.info(_("PyTone %s startup") % version.version)
+
+import errors
 import mainscreen
 import helper
 import hub, events
-import log
 import services.songdb
 import services.player
 import services.timer
 
-# Uncomment the next line, if you want to experience a little bit with
+# Uncomment the next line, if you want to experiment a little bit with
 # the number of bytecode instructions after which a context switch
 # occurs.
 
@@ -92,14 +102,25 @@ try:
     # there periodic events with this service
     services.timer.timer().start()
 
+    # initialize song database manager and start it immediately so
+    # that it can propagate quit events in case something goes wrong
+    # when setting up the databases
+    songdbmanager = services.songdb.songdbmanager()
+    songdbmanager.start()
+
     # song databases
     songdbids = []
-    songdbid = services.songdb.initsongdb("main", config.database.main)
-    if songdbid:
-        songdbids.append(songdbid)
-    songdbid = services.songdb.initsongdb("secondary", config.database.secondary)
-    if songdbid:
-        songdbids.append(songdbid)
+    for songdbname in config.database.getsubsections():
+        try:
+            songdbid = songdbmanager.addsongdb(songdbname, config.database[songdbname])
+            if songdbid:
+                songdbids.append(songdbid)
+        except Exception, e:
+            log.error("cannot initialize db %s: %s" % (id, e))
+
+    if not songdbids:
+        # raise last configuration error
+        raise 
 
     # network service
     if config.network.enableserver:
@@ -109,13 +130,36 @@ try:
         import network
         network.unixserver(os.path.expanduser(config.network.socketfile)).start()
 
-    # Now that everything is setup, we can initialize the players. This has
-    # to be done after all other services have been setup because
-    # the players immediately start requesting a new song
+    # Now that the basic services have been started, we can initialize
+    # the players. This has to be done last because the players
+    # immediately start requesting a new song
     playerids = [services.player.initplayer("main", config.player.main),
                  services.player.initplayer("secondary", config.player.secondary)]
+
+    # Determine plugins specified in the config file and read their config.
+    # The result goees into a list of tuples (pluginmodule, pluginconfig).
+    plugins = []
+
+    # Extend search path for plugins
+    sys.path[0:0] = [os.path.expanduser('~/.pytone')]
+
+    for name in config.general.plugins:
+        # import plugin module, taking into account that __import__ returns
+        # the top-level package, when using it with a dotted path
+        pluginmodule = getattr(__import__("plugins.%s" % name), name)
+
+        # process configuration of plugin
+        pluginconfig = pluginmodule.config
+        if pluginconfig is not None:
+            config.readconfigsection("plugin.%s" % name, pluginconfig)
+            config.finishconfigsection(pluginconfig)
+        pluginconfig = pluginconfig()
+        plugins.append((pluginmodule, pluginconfig))
+
+    # Set sys.path back to its original state
+    del sys.path[0]
 except:
-    # shutdown all already running services
+    # if something goes wrong, shutdown all already running services
     hub.notify(events.quit(), 100)
     raise
 
@@ -183,6 +227,12 @@ def cursessetup():
     # enable all mouse events
     curses.mousemask(curses.ALL_MOUSE_EVENTS)
 
+    # redirect stderr to /dev/null (to prevent spoiling the screen
+    # with libalsa messages). This is not really nice but at the moment there
+    # is no other way to get rid of these nuisance.
+    dev_null = file("/dev/null", 'w')
+    os.dup2(dev_null.fileno(), sys.stderr.fileno())
+
     return stdscr
 
 ##############################################################################
@@ -196,7 +246,7 @@ def cursescleanup():
         curses.echo()
         curses.nocbreak()
         curses.endwin()
-    except: 
+    except:
         pass
 
 
@@ -219,14 +269,14 @@ try:
     # set m to None as marker in case that something goes wrong in the
     # mainscreen.mainscreen constructor
     m = None
-    m = mainscreen.mainscreen(stdscr, songdbids, playerids)
+    m = mainscreen.mainscreen(stdscr, songdbids, playerids, plugins)
     m.run()
 except:
     cursescleanup()
-    
+
     # shutdown all other threads
     hub.notify(events.quit(), 100)
-    
+
     helper.print_exc_plus()
     raise
 else:
