@@ -64,6 +64,7 @@ def usage():
     print "    getplayerinfo:                  show information on the song currently being played"
     print "    playerforward:                  play the next song in the playlist"
     print "    playerprevious:                 play the previous song in the playlist"
+    print "    playerseekrelative <seconds>:   seek relative in the current song by the given number of seconds"
     print "    playerpause:                    pause the player"
     print "    playerstart:                    start/unpause the player"
     print "    playertogglepause:              pause the player, if playing, or play, if paused"
@@ -117,7 +118,7 @@ if len(args)==0:
     sys.exit(2)
 elif len(args)==1:
     if args[0]=="playerforward":
-        channel.notify(events.playerforward("main"))
+        channel.notify(events.playernext("main"))
     elif args[0]=="playerprevious":
         channel.notify(events.playerprevious("main"))
     elif args[0]=="playerpause":
@@ -156,6 +157,13 @@ else:
             usage()
             sys.exit(2)
         channel.notify(events.playerratecurrentsong("main", rating))
+    if args[0]=="playerseekrelative" and len(args)==2:
+        try:
+            seconds = float(args[1])
+        except:
+            usage()
+            sys.exit(2)
+        channel.notify(events.playerseekrelative("main", seconds))
     elif args[0]=="playlistaddsongs":
 	songs = [channel.request(requests.queryregistersong("main", path)) for path in args[1:]]
         channel.notify(events.playlistaddsongs(songs))
