@@ -17,12 +17,11 @@
 # along with PyTone; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import copy
-import time
-import threading
+import copy, time
 
 import events, hub, requests
 import services.playlist
+import service
 
 def initplayer(id, config):
     """ initialize player with id defined by config
@@ -158,8 +157,6 @@ class playbackinfo:
             s = s+ " (crossfading)"
         return s
 
-    __repr__ = __str__
-
     def updatesong(self, song):
         """ update song and reset time """
         self.song = song
@@ -196,7 +193,7 @@ class playbackinfo:
         return self.crossfade and not self.state == STOP
 
 
-class genericplayer(threading.Thread):
+class genericplayer(service.service):
     def __init__(self, id, playlistid, autoplay):
         """create a new player
         
@@ -207,11 +204,12 @@ class genericplayer(threading.Thread):
                     and it has not been stopped explicitely by the user
                     
         """
+        service.service.__init__(self, "player %s" % id, daemonize=True)
         self.id = id
         self.autoplay = autoplay
         self.playlistid = playlistid
         
-        # if wantplay!=autoplay, the user has requested a player stop and thus
+        # if wantplay != autoplay, the user has requested a player stop and thus
         # autoplay is effectively turned off, until the player is restarted again
         self.wantplay = autoplay
 
@@ -221,45 +219,47 @@ class genericplayer(threading.Thread):
         # old playbackinfo, used to detect changes of the player state
         self.oplaybackinfo = copy.copy(self.playbackinfo)
 
-        # as independent thread, we want our own event channel
-        self.channel = hub.newchannel()
-
-        self.done = False
-
-        threading.Thread.__init__(self)
-
-        # we cannot live without the rest
-        self.setDaemon(1)
-
         self.channel.subscribe(events.playerstart, self.playerstart)
         self.channel.subscribe(events.playerpause, self.playerpause)
         self.channel.subscribe(events.playertogglepause, self.playertogglepause)
         self.channel.subscribe(events.playerstop, self.playerstop)
-        self.channel.subscribe(events.playerforward, self.playerforward)
+        self.channel.subscribe(events.playernext, self.playernext)
         self.channel.subscribe(events.playerprevious, self.playerprevious)
-        self.channel.subscribe(events.nextsong, self.nextsong)
+        self.channel.subscribe(events.playerseekrelative, self.playerseekrelative)
+        self.channel.subscribe(events.playerplaysong, self.playerplaysong)
         self.channel.subscribe(events.playerratecurrentsong, self.playerratecurrentsong)
-        self.channel.subscribe(events.quit, self.quit)
         self.channel.supply(requests.getplaybackinfo, self.getplaybackinfo)
 
-    def run(self):
-        """main loop"""
-        while not self.done:
-            if self.isplaying():
-                self.play()
-            self.updatestatus()
-            
-            # request a new song, if none is playing and the player wants to play
-            if self.isstopped() and self.wantplay:
-                self.requestnextsong()
+    def work(self):
+        if self.isplaying():
+            self.play()
 
-            # wait some time if we do not have to do anything
-            if not self.isplaying():
-                self._playerreleasedevice()
-                time.sleep(0.5)
 
-            # process incoming events
-            self.channel.process()
+        # request a new song, if none is playing and the player wants to play
+        if self.isstopped() and self.wantplay:
+            self.requestnextsong()
+
+        # release player device if there is nothing to play
+        if not self.isplaying():
+            self._playerreleasedevice()
+
+        # process incoming events
+        self.channel.process()
+        
+        # and notify the rest of any changes in the playback status
+        self.updatestatus()
+        # Now the queue of all pending events has been
+        # cleared. Depending on the player status we can now wait for
+        # further incoming events.
+        if not self.isplaying():
+            # We sleep a little bit to prevent being overly active
+            # when the event channel is spilled by messages
+            time.sleep(0.2)
+            # In this case, we can safely block since we will be waked
+            # up by any message on the event channel. Thus, event if
+            # we want to request a new song, we can rely on an event
+            # signaling the addition of a new song to the playlist
+            self.channel.process(block=True)
 
     def play(self):
         """play songs
@@ -285,8 +285,8 @@ class genericplayer(threading.Thread):
         manual indicates whether the user has requested the song manually
         """
         if song:
-            self.playbackinfo.playing()
             self._playsong(song, manual)
+            self.playbackinfo.playing()
 
     def isstopped(self):
         return self.playbackinfo.state == STOP
@@ -328,6 +328,12 @@ class genericplayer(threading.Thread):
 
         this method has to be implemented by specialized classes"""
         pass
+    
+    def _playerseekrelative(self, seconds):
+        """seek by the given number of seconds in file (relative to current position)
+
+        this method has to be implemented by specialized classes"""
+        pass
 
     def _playerreleasedevice(self):
         """temporarily release audio device 
@@ -378,7 +384,7 @@ class genericplayer(threading.Thread):
             self.playbackinfo.stopped()
             self._playerstop()
 
-    def playerforward(self, event):
+    def playernext(self, event):
         """immediately play next song"""
         if event.playerid == self.id:
             self.requestnextsong(manual=1)
@@ -388,7 +394,12 @@ class genericplayer(threading.Thread):
         if event.playerid == self.id:
             self.requestnextsong(manual=1, previous=1)
 
-    def nextsong(self, event):
+    def playerseekrelative(self, event):
+        """seek by event.seconds in file (relative to current position """
+        if event.playerid == self.id:
+            self._playerseekrelative(event.seconds)
+
+    def playerplaysong(self, event):
         """play event.song next"""
         if event.playerid == self.id:
             self.playsong(event.song, manual=1)
@@ -400,7 +411,7 @@ class genericplayer(threading.Thread):
 
     def quit(self, event):
         """quit player"""
-        self.done = True
+        service.service.quit(self, event)
         self._playerquit()
 
     # request handlers
