@@ -65,7 +65,7 @@ class bufferedaudiodev(threading.Thread):
         self.aooptions = aooptions
         self.rate = rate
         self.SIZE = SIZE
-
+        
         # initially, we do not open the audio device
         self.audiodev = None
 
@@ -78,9 +78,7 @@ class bufferedaudiodev(threading.Thread):
         self.ispaused = False
         self.restart = threading.Event()
         threading.Thread.__init__(self)
-
-        # we don't know what to do, if we are left alone, hence:
-        self.setDaemon(1)
+        self.setDaemon(True)
 
     def opendevice(self):
         errorlogged = False
@@ -104,9 +102,15 @@ class bufferedaudiodev(threading.Thread):
 
     def closedevice(self):
         if self.audiodev is not None:
-            self.audiodev.close()
-            log.debug("audio device closed")
+            # we use self.audiodev = None as a marker for a closed audio device
+            # To avoid race conditions we thus first have to mark the device as
+            # closed before really closing it. Note that when we try to reopen the device
+            # later, and it has not yet been closed, the opendevice method will retry
+            # this 
+            openaudiodev = self.audiodev
             self.audiodev = None
+            openaudiodev.close()
+            log.debug("audio device closed")
 
     def queuelen(self):
         """ return length of currently buffered PCM data in seconds"""
@@ -114,31 +118,28 @@ class bufferedaudiodev(threading.Thread):
 
     def run(self):
         while not self.done:
-            if self.ispaused:
-                self.restart.wait()
-                self.restart.clear()
-            buff, bytes = self.queue.get(1)
-            if self.audiodev is None:
-                self.opendevice()
-            if buff != 0 and bytes != 0:
-                self.audiodev.play(buff, bytes)
+            try:
+                if self.ispaused:
+                    self.restart.wait()
+                    self.restart.clear()
+                buff, bytes = self.queue.get(1)
+                if self.audiodev is None:
+                    self.opendevice()
+                if buff != 0 and bytes != 0:
+                    self.audiodev.play(buff, bytes)
+            except:
+                log.warning("exception occured in bufferedaudiodev")
+                log.debug_traceback()
 
     def play(self, buff, bytes):
         self.queue.put((buff, bytes))
-
+        
     def flush(self):
-        self.ispaused = True
         while True:
             try:
                 self.queue.get(0)
             except Queue.Empty:
                 break
-        self.play(0, 0)
-        self.closedevice()
-
-    def stop(self):
-        self.done = True
-        self.flush()
 
     def pause(self):
         self.ispaused = True
@@ -148,6 +149,12 @@ class bufferedaudiodev(threading.Thread):
         self.ispaused = False
         self.restart.set()
 
+    def quit(self):
+        self.done = True
+        self.flush()
+        self.closedevice()
+        self.restart.set()
+
 
 class player(genericplayer):
 
@@ -177,11 +184,17 @@ class player(genericplayer):
 
         genericplayer.__init__(self, id, playlistid, autoplay)
 
+    def _flushqueue(self):
+        """ delete internal player queue and flush audiodevice """
+        self.songs = []
+        self.audiodev.flush()
+        self.audiodev.closedevice()
+
     def play(self):
         """decode songs and mix them together"""
 
         # unpause buffered ao if necessary
-        if isinstance(self.audiodev, bufferedaudiodev) and self.audiodev.ispaused:
+        if self.audiodev.ispaused:
             self.audiodev.unpause()
 
         if len(self.songs) == 1:
@@ -192,11 +205,11 @@ class player(genericplayer):
             else:
                 log.debug("internal player: song ends: %s (0 songs in queue)" % self.songs[0])
                 del self.songs[0]
-
+        
             # reset songtransition mode, but before possibly requesting a new song
             self.songtransitionmode = None
 
-            if self.crossfading and song.ttime-song.ptime<self.crossfadingstart: 
+            if len(buff) == 0 or (self.crossfading and song.ttime-song.ptime < self.crossfadingstart):
                 self.requestnextsong()
 
         elif len(self.songs) == 2:
@@ -229,7 +242,7 @@ class player(genericplayer):
             elif self.songtransitionmode == "gapkill":
                 # just kill gap between songs
                 buff = self.songs[0].read(self.SIZE)
-                if len(buff)<self.SIZE:
+                if len(buff) < self.SIZE:
                     log.debug("internal player: song ends: %s (1 song in queue)" % self.songs[0])
                     del self.songs[0]
 
@@ -270,6 +283,9 @@ class player(genericplayer):
     def _playsong(self, song, manual):
         log.debug("internal player: new song: %s" % song)
 
+        if self.ispaused():
+            self._flushqueue()
+
         # we want maximally 2 songs in queue
         if len(self.songs) == 2:
             del self.songs[0]
@@ -298,13 +314,25 @@ class player(genericplayer):
         log.debug("internal player: %d songs in queue" % len(self.songs))
 
     def _playerpause(self):
-        if isinstance(self.audiodev, bufferedaudiodev):
-            self.audiodev.pause()
+        self.audiodev.pause()
 
     def _playerstop(self):
-        self.songs = []
-        if isinstance(self.audiodev, bufferedaudiodev):
-            self.audiodev.flush()
+        self._flushqueue()
+
+    def _playerseekrelative(self, seconds):
+        if len(self.songs) == 0:
+            return
+        if len(self.songs) == 2:
+            # we refuse to seek backward during crossfading
+            if seconds < 0:
+                return
+            del self.songs[0]
+        song = self.songs[0]
+        song.seekrelative(seconds)
+        self.audiodev.flush()
 
     def _playerreleasedevice(self):
         self.audiodev.closedevice()
+
+    def _playerquit(self):
+        self.audiodev.quit()
