@@ -17,21 +17,18 @@
 # along with PyTone; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import copy, threading
+import copy, service
 import hub, events, requests
 import network
 
-class player(threading.Thread):
+class player(service.service):
     def __init__(self, id, server, port):
-        threading.Thread.__init__(self)
+        service.service.__init__(self, "remote player")
         self.id = id
         self.server = server
         self.port = port
         self.remoteplayerid = "main"
 
-        # as independent thread, we want our own event channel
-        self.channel = hub.newchannel()
-
         self.networkchannel = network.tcpclientchannel(self.server, self.port)
         #self.networkchannel.transmit(events.updatesong)
         #self.networkchannel.transmit(events.updatealbum)
@@ -43,32 +40,15 @@ class player(threading.Thread):
         self.networkchannel.subscribe(events.playlistchanged, self.playlistchanged)
         self.networkchannel.start()
 
-        self.done = False
-
-        # we are only servants of higher powers
-        self.setDaemon(1)
-
         # provide player service
         self.channel.subscribe(events.playerevent, self.playerevent)
-        # self.channel.subscribe(events.nextsong, self.playerevents)
         
         # we also provide a playlist service
         self.channel.subscribe(events.playlistevent, self.playlistevent)
         self.channel.supply(requests.playlistgetcontents, self.playlistgetcontents)
         
-        self.channel.subscribe(events.quit, self.quit)
-
-
-    def run(self):
-        """main loop"""
-        while not self.done:
-            self.channel.process(blocking=True)
-
     # event handler
 
-    def quit(self, event):
-        self.done = True
-
     def playbackinfochanged(self, event):
         # this is called by the networkchannel, so it should be (and is) thread safe
         event.playerid = self.id
