@@ -20,45 +20,44 @@
 import os.path
 import random
 import time
-import threading
 import pickle
 
 import config
 import events, hub, requests
 import dbitem, item
 import log
+import service
 
 _counter = 0
 
 class playlistitem:
     """ wrapped song with the two additional attributes played and id
-    
-    - played: has playlist item already been played
+
     - id:     unique id for playlist item (note that the same song
-              can be present more than once in the playlist)"""
-    
-    def __init__(self, song):
+              can be present more than once in the playlist)
+    - played: has playlist item already been played
+    - playstarttime: time at which song has been played or None if played is False
+    """
+
+    def __init__(self, song, played=False, playstarttime=None):
         global _counter
         self.song = song
-        self.played = False
-        self.playstarttime = None
+        self.played = played
+        self.playstarttime = playstarttime
         self.id = _counter
         _counter += 1
 
     def __repr__(self):
         return "playlistitem: id=%s" % `self.id`
 
-    def __cmp__(self, other):
-        try:
-            return cmp(self.id, other.id)
-        except:
-            return 1
+    def getid(self):
+        return self.id
 
-    def __getattr__(self, attr):
-        # Python tries to call __setstate__ upon unpickling -- prevent this
-        if attr=="__setstate__":
-            raise AttributeError
-        return getattr(self.song, attr)
+    def getinfo(self):
+        return self.song.getinfo()
+
+    def getinfolong(self):
+        return self.song.getinfolong()
 
     def markplayed(self):
         self.played = True
@@ -78,29 +77,23 @@ def initplaylist(id, playerid, songdbid):
     playlist(id, playerid, songdbid).start()
 
 
-class playlist(threading.Thread):
+class playlist(service.service):
     """manage playlist for a single player, which can be accessed 
     by multipled users"""
 
     def __init__(self, id, playerid, songdbid):
-        threading.Thread.__init__(self)
-        self.setName("playlist thread")
+        service.service.__init__(self, "playlist")
         self.id = id
         # each playlist service is identified by the corresponding player
         self.playerid = playerid
         self.songdbid = songdbid
-        self.songs = []
+        self.items = []
         self.ttime = 0
         self.ptime = 0
-        self.playingsong = None
+        self.playingitem = None
         self.logfilename = config.general.logfile
         self.autoplaymode = config.general.autoplaymode
 
-        # as independent thread, we want our own event and request channel
-        self.channel = hub.newchannel()
-
-        self.done = False
-        
         self.channel.subscribe(events.playbackinfochanged, self.playbackinfochanged)
         self.channel.subscribe(events.playerstop, self.playerstop)
         self.channel.subscribe(events.playlistaddsongs, self.playlistaddsongs)
@@ -116,12 +109,12 @@ class playlist(threading.Thread):
         self.channel.subscribe(events.playlistsave, self.playlistsave)
         self.channel.subscribe(events.playlistshuffle, self.playlistshuffle)
         self.channel.subscribe(events.playlisttoggleautoplaymode, self.playlisttoggleautoplaymode)
+        self.channel.subscribe(events.playlistplaysong, self.playlistplaysong)
         self.channel.subscribe(events.songchanged, self.songchanged)
-        self.channel.subscribe(events.quit, self.quit)
 
         self.channel.supply(requests.requestnextsong, self.requestnextsong)
         self.channel.supply(requests.playlistgetcontents, self.playlistgetcontents)
-        
+
         # try to load dump from prior crash, if existent
         if config.general.dumpfile:
             try:
@@ -131,78 +124,83 @@ class playlist(threading.Thread):
             except:
                 pass
 
-    def run(self):
-        """main loop"""
-        while not self.done:
-            self.channel.process(blocking=True)
-
     def append(self, item):
-        # TODO: take crossfading time into account
-        if not item.hasbeenplayed():
-            item.playstarttime = time.time()+self.ttime-self.ptime
-        self.ttime += item.length
+        self.ttime += item.song.length
         if item.hasbeenplayed():
-            self.ptime += item.length
-        self.songs.append(item)
+            self.ptime += item.song.length
+        self.items.append(item)
 
     def insert(self, index, item):
-        self.ttime += item.length
+        self.ttime += item.song.length
         if item.hasbeenplayed():
-            self.ptime += item.length
-        self.songs.insert(index, item)
-        self._updateplaystarttimes()
+            self.ptime += item.song.length
+        self.items.insert(index, item)
 
     def __delitem__(self, index):
-        item = self.songs[index]
-        self.ttime -= item.length
+        item = self.items[index]
+        self.ttime -= item.song.length
         if item.hasbeenplayed():
-            self.ptime -= item.length
-        self.songs.__delitem__(index)
+            self.ptime -= item.song.length
+        self.items.__delitem__(index)
+        self._updateplaystarttimes()
 
-    def _notifyplaylistchanged(self):
-        hub.notify(events.playlistchanged(self.songs, self.ptime, self.ttime, self.autoplaymode, self.playingsong))
+    # all methods starting with an underscore may modify the playlist but leave
+    # it up to the caller to announce this change via an playlistchanged event
 
-    def _searchnextsong(self):
+    def _searchnextitem(self):
         """return playlistitem which has to be played next or None"""
-        for i in range(len(self.songs)):
-            if not self.songs[i].hasbeenplayed():
-                return self.songs[i]
+        for i in range(len(self.items)):
+            if not self.items[i].hasbeenplayed():
+                return self.items[i]
         return None
 
-    def _logplay(self, song):
+    def _logplay(self, item):
         if self.logfilename:
             logfile = open(self.logfilename, "a")
-            logfile.write("%s: %s\n" % (time.asctime(), song.path))
+            logfile.write("%s: %s\n" % (time.asctime(), item.song.path))
             logfile.close()
 
+    def _checkabortion(self):
+        """check whether currently playing song was aborted too early"""
+        if self.playingitem:
+            playingtime = time.time() - self.playingitem.playstarttime
+            # check whether the song has been played at least 
+            # 30 seconds or alternatively for 80 percent of its total
+            # length
+            if playingtime < min(30, 0.8*self.playingitem.song.length):
+                # if not, consider it as not having been played
+                self.playingitem.song.unplay()
+
     def _updateplaystarttimes(self):
-        if self.playingsong:
-            playstarttime = self.playingsong.playstarttime + self.playingsong.length
-            playingsongcounted = True
+        # TODO: take crossfading time into account
+        if self.playingitem:
+            playstarttime = self.playingitem.playstarttime + self.playingitem.song.length
         else:
             playstarttime = time.time()
-            playingsongcounted = False
-        for song in self.songs:
-            if not song.hasbeenplayed():
-                song.playstarttime = playstarttime
-                playstarttime += song.length
+        for item in self.items:
+            if not item.hasbeenplayed():
+                item.playstarttime = playstarttime
+                playstarttime += item.song.length
+
+    def _playitem(self, item):
+        """ check for a song abortion, register song as being played
+        and update playlist information accordingly"""
+        
+        self._checkabortion()
+        if not item.hasbeenplayed():
+            self.ptime += item.song.length
+        item.markplayed()
+        self.playingitem = item
+        self._updateplaystarttimes()
+        self._logplay(item)
 
     def _playnext(self):
         """ mark next item from playlist as played and as currently playing and return
         corresponding song"""
-        nextsong = self._searchnextsong()
-        if nextsong:
-            # check whether currently playing song was aborted too early
-            if self.playingsong and time.time()-self.playingsong.playstarttime<min(10, self.playingsong.length-1):
-                # if yes, consider it as not having been played
-                self.playingsong.unplay()
-            self.ptime += nextsong.length
-            nextsong.markplayed()
-            self.playingsong = nextsong
-            self._updateplaystarttimes()
-            self._notifyplaylistchanged()
-            self._logplay(nextsong)
-            return nextsong
+        nextitem = self._searchnextitem()
+        if nextitem:
+            self._playitem(nextitem)
+            return nextitem
         else:
             return None
 
@@ -210,47 +208,33 @@ class playlist(threading.Thread):
         """ mark next item from playlist as played & currently playing and return
         corresponding song"""
 
-        if self.playingsong:
-            currentsong = self.playingsong
+        # start either from the currently playing song, or if no song
+        # is currently played, the first unplayed song in the playlist...
+        #
+        if self.playingitem:
+            currentitem = self.playingitem
         else:
-            currentsong = self._searchnextsong()
-            
-        if currentsong:
-            i = self.songs.index(currentsong)
+            currentitem = self._searchnextitem()
+
+        if currentitem:
+            # ... and go back one song
+            i = self.items.index(currentitem)
             if i == 0:
                 return
-
-            # check whether currently playing song was aborted too early
-            if self.playingsong and time.time()-self.playingsong.playstarttime < min(10, self.playingsong.length-1):
-                # if yes, consider it as not having been played
-                if self.playingsong.nrplayed == 0:
-                    log.info("%s, %d" % (self.playingsong.path, self.playingsong.nrplayed))
-                else:
-                    self.playingsong.unplay()
-            
-            if currentsong.hasbeenplayed():
-                currentsong.markunplayed()
-                self.ptime -= currentsong.length
-
-            song = self.songs[i-1]
-            if not song.hasbeenplayed():
-                self.ptime += song.length
-            song.markplayed()
-            self.playingsong = song
-            self._logplay(song)
-            self._updateplaystarttimes()
-            self._notifyplaylistchanged()
-            return song
+            self._markunplayed(currentitem)
+            item = self.items[i-1]
+            self._playitem(item)
+            return item
 
     def _clear(self):
-        self.songs = []
+        self.items = []
         self.ptime = 0
         self.ttime = 0
-        self.playingsong = None
+        self.playingitem = None
 
     def _deleteplayedsongs(self):
-        for i in range(len(self.songs)-1,-1,-1):
-            if self.songs[i].hasbeenplayed() and self.songs[i] != self.playingsong:
+        for i in range(len(self.items)-1,-1,-1):
+            if self.items[i].hasbeenplayed() and self.items[i] != self.playingitem:
                 del self[i]
 
     def _checksong(self, song):
@@ -263,17 +247,16 @@ class playlist(threading.Thread):
                 return song
         if os.path.isfile(song.path):
             # first we try to access the song via its filesystem path
-            asong = hub.request(requests.queryregistersong(self.songdbid, song.path))
-            return item.song(self.songdbid, song=asong)
+            return hub.request(requests.queryregistersong(self.songdbid, song.path))
 
-        if song.artist!=dbitem.UNKNOWN and song.album!=dbitem.UNKNOWN:
+        if song.artist != dbitem.UNKNOWN and song.album != dbitem.UNKNOWN:
             # otherwise we use the artist and album tags and try to obtain the song via
             # the database
             songs = hub.request(requests.getsongs(self.songdbid,
-                                                      artist=song.artist, album=song.album))
+                                                  artist=song.artist, album=song.album))
             for asong in songs:
-                if asong.title==song.title:
-                    return item.song(self.songdbid, song=asong)
+                if asong.title == song.title:
+                    return asong
 
         # song not found
         # XXX start transmitting song
@@ -286,17 +269,25 @@ class playlist(threading.Thread):
                 song = self._checksong(song)
                 if song:
                     self.append(playlistitem(song))
-        self._notifyplaylistchanged()
+        self._updateplaystarttimes()
+
+    def _markunplayed(self, item):
+        """ mark song unplayed and adjust playlist information accordingly """
+        if item.hasbeenplayed():
+            self.ptime -= item.song.length
+            item.markunplayed()
+            self._updateplaystarttimes()
 
     def _markallunplayed(self):
         """ mark all songs in playlist as not having been played """
-        for song in self.songs:
-            if song.hasbeenplayed():
-                song.markunplayed()
-                self.ptime -= song.length
-        self._updateplaystarttimes()
-        self._notifyplaylistchanged()
+        for item in self.items:
+            self._markunplayed(item)
 
+    # convenience method for issuing a playlistchanged event
+    
+    def notifyplaylistchanged(self):
+        hub.notify(events.playlistchanged(self.items, self.ptime, self.ttime, self.autoplaymode, self.playingitem))
+    
     # statusbar input handler
 
     def saveplaylisthandler(self, name, key):
@@ -306,143 +297,151 @@ class playlist(threading.Thread):
                 name = name + ".m3u"
             try:
                 file = open(name, "w")
-                for song in self.songs:
-                    file.write("%s\n" % song.path)
+                for item in self.items:
+                    file.write("%s\n" % item.song.path)
                 file.close()
-                hub.notify(events.registerplaylists(self.songdbid, [name]))
+                playlist = dbitem.playlist(name)
+                hub.notify(events.registerplaylists(self.songdbid, [playlist]))
             except (IOError, OSError):
                 pass
 
     def loadplaylisthandler(self, name, key):
-        if key==ord("\n"):
-            if name[-4:]!=".m3u":
+        if key == ord("\n"):
+            if name[-4:] != ".m3u":
                 name = name + ".m3u"
             try:
-                songs = []
                 path = os.path.join(config.general.playlistdir, name)
                 file = open(path, "r")
                 self._clear()
                 for line in file.xreadlines():
                     if not line.startswith("#"):
-                        dbsong = hub.request(requests.queryregistersong(self.songdbid,
-                                                                        line.strip()))
-                        if dbsong:
-                            self.append(playlistitem(item.song(self.songdbid, song=dbsong)))
+                        song = hub.request(requests.queryregistersong(self.songdbid,
+                                                                      line.strip()))
+                        if song:
+                            self.append(playlistitem(song))
                 file.close()
             except (IOError, OSError):
                 pass
-            self._notifyplaylistchanged()
+            self._updateplaystarttimes()
+            self.notifyplaylistchanged()
 
     def _locatesong(self, id):
-        """ locate position of song in playlist by id """
-        for song, i in zip(self.songs, range(len(self.songs))):
-            if song.id==id:
+        """ locate position of item in playlist by id """
+        for item, i in zip(self.items, range(len(self.items))):
+            if item.id == id:
                 return i
         else:
             return None
 
     def dump(self):
         """ write playlist to dump file """
-        if self.playingsong:
-            self.playingsong.markunplayed()
-        self._deleteplayedsongs()
-        self._notifyplaylistchanged()
+        if self.playingitem:
+            self.playingitem.markunplayed()
+        # self._deleteplayedsongs()
+        self.notifyplaylistchanged()
         dumpfile = open(config.general.dumpfile, "w")
-        pickle.dump(self.songs, dumpfile)
+        pickle.dump(self.items, dumpfile)
 
     def load(self):
         """ load playlist from file """
         dumpfile = open(config.general.dumpfile, "r")
         self._clear()
-        songs = []
         for item in pickle.load(dumpfile):
             # We have to be careful here and not use the playlist item
             # stored in the dump file directly, since its id and the
             # global _counter variable are not in accordance. Besides that
             # the playstarttime information stored in the pickle is incorrect.
             # We thus have to create a new playlistitem.
-            self.append(playlistitem(item.song))
+            newplaylistitem = playlistitem(item.song, item.played, item.playstarttime)
+            self.append(newplaylistitem)
+        self._updateplaystarttimes()
 
     # event handlers
 
     def playbackinfochanged(self, event):
-       # We are only interested in the case of the player having been stopped (because
-       # there is no more song in the playlist).
+       # We are only interested in the case of the player having been stopped due to
+       # no more song being left in the playlist.
        if event.playbackinfo.isstopped():
-           self.playingsong = None
-           self._notifyplaylistchanged()
+           self.playingitem = None
+           self.notifyplaylistchanged()
 
     def playerstop(self, event):
+        # Mark the currently playing song as unplayed again when the
+        # player has been stopped manually. Note that the handling of
+        # this event is potentially racy with the playbackinfochanged
+        # event, but the ordering of the events in our event channel
+        # should prevent any problems.
         if event.playerid == self.playerid:
-            if self.playingsong:
-                self.playingsong.markunplayed()
-                self.playingsong = None
-                self._notifyplaylistchanged()
-
+            if self.playingitem:
+                self._checkabortion()
+                self._markunplayed(self.playingitem)
+                self.playingitem = None
+                self.notifyplaylistchanged()
 
     def playlistaddsongs(self, event):
         self._addsongs(event.songs)
+        self.notifyplaylistchanged()
 
     def playlistaddsongtop(self, event):
         if event.song:
             song = self._checksong(event.song)
             if song:
-                newsong = playlistitem(song)
-                for i in range(len(self.songs)):
-                    if not self.songs[i].hasbeenplayed():
-                        self.insert(i, newsong)
+                newitem = playlistitem(song)
+                for i in range(len(self.items)):
+                    if not self.items[i].hasbeenplayed():
+                        self.insert(i, newitem)
                         break
                 else:
-                    self.append(newsong)
-                self._updateplaystarttimes()
-                self._notifyplaylistchanged()
-                hub.notify(events.nextsong(self.playerid, self._playnext()))
+                    self.append(newitem)
+                hub.notify(events.playerplaysong(self.playerid, self._playnext()))
+                self.notifyplaylistchanged()
 
     def playlistdeletesong(self, event):
         i = self._locatesong(event.id)
         if i is not None:
             del self[i]
-            self._updateplaystarttimes()
-            self._notifyplaylistchanged()
+            self.notifyplaylistchanged()
 
     def playlistmovesongup(self, event):
         i = self._locatesong(event.id)
         if i is not None and i > 0:
-            self.songs[i-1], self.songs[i] = self.songs[i], self.songs[i-1]
+            self.items[i-1], self.items[i] = self.items[i], self.items[i-1]
             self._updateplaystarttimes()
-            self._notifyplaylistchanged()
+            self.notifyplaylistchanged()
 
     def playlistmovesongdown(self, event):
         i = self._locatesong(event.id)
-        if i is not None and i<len(self.songs)-1:
-            self.songs[i], self.songs[i+1] = self.songs[i+1], self.songs[i]
+        if i is not None and i<len(self.items)-1:
+            self.items[i], self.items[i+1] = self.items[i+1], self.items[i]
             self._updateplaystarttimes()
-            self._notifyplaylistchanged()
+            self.notifyplaylistchanged()
 
     def playlistclear(self, event):
         self._clear()
-        self._notifyplaylistchanged()
-        
+        self.notifyplaylistchanged()
+
     def playlistdeleteplayedsongs(self, event):
         self._deleteplayedsongs()
-        self._notifyplaylistchanged()
+        self.notifyplaylistchanged()
 
     def playlistreplay(self, event):
         self._markallunplayed()
+        self.notifyplaylistchanged()
 
     def playlistsave(self, event):
         hub.notify(events.requestinput(_("Save playlist"),
-                                              _("Name:"),
-                                             self.saveplaylisthandler))
+                                       _("Name:"),
+                                       self.saveplaylisthandler))
 
     def playlistload(self, event):
         hub.notify(events.requestinput(_("Load playlist"),
-                                              _("Name:"),
-                                             self.loadplaylisthandler))
+                                       _("Name:"),
+                                       self.loadplaylisthandler))
 
     def playlistshuffle(self, event):
-        random.shuffle(self.songs)
-        self._notifyplaylistchanged()
+        random.shuffle(self.items)
+        self._updateplaystarttimes()
+        self.notifyplaylistchanged()
 
     def playlisttoggleautoplaymode(self, event):
         if self.autoplaymode == "off":
@@ -451,21 +450,27 @@ class playlist(threading.Thread):
             self.autoplaymode = "random"
         else:
             self.autoplaymode = "off"
-        self._notifyplaylistchanged()
+        self.notifyplaylistchanged()
+
+    def playlistplaysong(self, event):
+        i = self._locatesong(event.id)
+        self._playitem(self.items[i])
+        self.notifyplaylistchanged()
+        hub.notify(events.playerplaysong(self.playerid, self.items[i].song))
 
     def songchanged(self, event):
         # check whether one of our playlist items is affected by the change
         # in the songdb
-        for item in self.songs:
+        for item in self.items:
             if item.song == event.song and item.song.id==event.songdbid:
                 item.song = event.song
                 self._updateplaystarttimes()
-                self._notfiyplaylistchanged()
+                self.notifyplaylistchanged()
 
     def quit(self, event):
+        service.service.quit(self, event)
         self.dump()
-        self.done = True
-        
+
     #
     # request handler
     #
@@ -474,20 +479,24 @@ class playlist(threading.Thread):
         if request.playlistid != self.id:
             raise hub.DenyRequest
         if not request.previous:
-            nextsong = self._playnext()
-            if not nextsong:
+            nextitem = self._playnext()
+            if not nextitem:
                 if self.autoplaymode == "random":
                     # add some randomly selected song to the end of the playlist
-                    randomsongs = hub.request(requests.getsongs(self.songdbid, random=True))
+                    randomsongs = hub.request(requests.getsongs(None, random=True))
                     if randomsongs:
                         self._addsongs(randomsongs[0:1])
-                        nextsong = self._playnext()
+                        nextitem = self._playnext()
                 elif self.autoplaymode == "repeat":
                     self._markallunplayed()
-                    nextsong = self._playnext()
+                    nextitem = self._playnext()
         else:
-            nextsong = self._playprevious()
-        return nextsong
+            nextitem = self._playprevious()
+        self.notifyplaylistchanged()
+        if nextitem:
+            return nextitem.song
+        else:
+            return None
 
     def playlistgetcontents(self, request):
-        return self.songs, self.ptime, self.ttime, self.autoplaymode, self.playingsong
+        return self.items, self.ptime, self.ttime, self.autoplaymode, self.playingitem
