@@ -17,21 +17,19 @@
 # along with PyTone; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import math
 import os
 import errno
-import random
-import threading
 import time
 import bsddb.dbshelve
 import bsddb.db
 
-import config
-import events, hub
+import events, hub, requests
 import errors
 import log
+import metadata
 import dbitem
-import requests
+import service
+
 
 class mydbshelve(bsddb.dbshelve.DBShelf):
     def __init__(self, filename, flags=bsddb.db.DB_CREATE, mode=0660, filetype=bsddb.db.DB_HASH, dbenv=None, dbname=None):
@@ -96,9 +94,6 @@ class dbenv:
         # Checkpoint the database twice, as recommended by Sleepycat
         self.checkpoint()
         self.checkpoint()
-        # delete log files (even when DB_LOG_AUTOREMOVE is not supported)
-        for logfile in self.dbenv.log_archive(bsddb.db.DB_ARCH_ABS):
-            os.unlink(logfile)
 
         self.dbenv.close()
         self.dbenv = None
@@ -114,7 +109,8 @@ class dbenv:
 
     def checkpoint(self):
         """flush memory pool, write checkpoint record to log and flush flog"""
-        # this sometimes results in an intermittent error (no idea why) 
+        # this sometimes results in an intermittent error (no idea why)
+        log.debug("checkpointing database")
         try:
             self.dbenv.txn_checkpoint(0, 0, bsddb.db.DB_FORCE)
         except bsddb._db.DBInvalidArgError:
@@ -124,7 +120,12 @@ class dbenv:
             except bsddb._db.DBInvalidArgError:
                 # if it still doesn't work, we don't feel responsible
                 # anymore and just log the error
-                log.info("checkpointing the database failed with DBInvalidArgError\n")
+                log.warning("checkpointing the database failed with DBInvalidArgError\n")
+                return
+
+        # delete log files (even when DB_LOG_AUTOREMOVE is not supported)
+        for logfile in self.dbenv.log_archive(bsddb.db.DB_ARCH_ABS):
+            os.unlink(logfile)
 
 #
 # songdb class
@@ -133,9 +134,9 @@ class dbenv:
 # interval in seconds after which logs are flushed
 checkpointinterval = 60
 
-class songdb(threading.Thread):
-    def __init__(self, id, config):
-        threading.Thread.__init__(self)
+class songdb(service.service):
+    def __init__(self, id, config, songdbhub):
+        service.service.__init__(self, "%s songdb" % id, hub=songdbhub)
         self.id = id
         self.songdbbase = config.basename
         self.dbfile = config.dbfile
@@ -147,7 +148,7 @@ class songdb(threading.Thread):
         self.tagremoveaccents = config.tags_removeaccents
 
         if not os.path.isdir(self.basedir):
-            raise errors.configurationerror("musicbasedir of database is not a directory.")
+            raise errors.configurationerror("musicbasedir '%s' of database %s is not a directory." % (self.basedir, self.id))
 
         if not os.access(self.basedir, os.X_OK | os.R_OK):
             raise errors.configurationerror("you are not allowed to access and read config.general.musicbasedir.")
@@ -162,33 +163,24 @@ class songdb(threading.Thread):
         try:
             self._initdb()
         except:
-            raise
             raise errors.databaseerror("cannot initialise/open song database files.")
 
-        # empty cache of all artists and all songs
-        # This variables contain tuples acting as caches for
-        # all artists and all songs, respectively, in the database
-        self.allartists = self.allsongs = None
-
-        # as independent thread, we want our own event and request channel
-        self.channel = hub.newchannel()
-
-        self.done = False
-
         # we need to be informed about database changes
         self.channel.subscribe(events.updatesong, self.updatesong)
         self.channel.subscribe(events.rescansong, self.rescansong)
         self.channel.subscribe(events.delsong, self.delsong)
+        self.channel.subscribe(events.updateplaylist, self.updateplaylist)
+        self.channel.subscribe(events.delplaylist, self.delplaylist)
         self.channel.subscribe(events.updatealbum, self.updatealbum)
         self.channel.subscribe(events.updateartist, self.updateartist)
         self.channel.subscribe(events.registersongs, self.registersongs)
         self.channel.subscribe(events.registerplaylists, self.registerplaylists)
         self.channel.subscribe(events.clearstats, self.clearstats)
-        self.channel.subscribe(events.quit, self.quit)
 
         # regularly flush the database log
-        self.channel.subscribe(events.checkpointdatabase, self.checkpointdatabase)
-        hub.notify(events.sendeventin(events.checkpointdatabase(self.id), checkpointinterval))
+        self.channel.subscribe(events.checkpointdb, self.checkpointdb)
+        # send this event to normal hub, since otherwise the timer service does not get it
+        hub.notify(events.sendeventin(events.checkpointdb(self.id), checkpointinterval, repeat=checkpointinterval))
 
         # we are a database service provider...
         self.channel.supply(requests.getdatabaseinfo, self.getdatabaseinfo)
@@ -216,7 +208,7 @@ class songdb(threading.Thread):
         self.channel.supply(requests.getsongsinplaylist, self.getsongsinplaylist)
         self.channel.supply(requests.getsongsinplaylists, self.getsongsinplaylists)
 
-        self.autoregisterer = songautoregisterer(self.basedir, self.id, self.channel,
+        self.autoregisterer = songautoregisterer(self.basedir, self.id, self.isbusy,
                                                  self.tracknrandtitlere,
                                                  self.tagcapitalize, self.tagstripleadingarticle, self.tagremoveaccents)
         self.autoregisterer.start()
@@ -503,9 +495,7 @@ class songdb(threading.Thread):
 
 
     def run(self):
-        """main loop"""
-        while not self.done:
-            self.channel.process(blocking=True)
+        service.service.run(self)
         self.close()
 
     def close(self):
@@ -563,7 +553,7 @@ class songdb(threading.Thread):
             log.debug("new album: %s" % song.album)
             album = dbitem.album(song.album)
             changed = True
-            hub.notify(events.albumaddedtosongdb(self.id, album))
+            # hub.notify(events.albumaddedordeleted(self.id, album))
 
         if song.id not in album.songs:
             changed = True
@@ -596,6 +586,7 @@ class songdb(threading.Thread):
             self.albums.put(song.album, album, txn=self.txn)
         else:
             self.albums.delete(song.album, txn=self.txn)
+            hub.notify(events.albumaddedordeleted(self.id, album))
 
     def _indexsong_artist(self, song):
         """ insert/update artist information for song """
@@ -608,7 +599,7 @@ class songdb(threading.Thread):
             log.debug("new artist: %s" % song.artist)
             artist = dbitem.artist(song.artist)
             changed = True
-            hub.notify(events.artistaddedtosongdb(self.id, artist))
+            hub.notify(events.artistaddedordeleted(self.id, artist))
 
         if song.album not in artist.albums:
             changed = True
@@ -619,9 +610,9 @@ class songdb(threading.Thread):
             artist.songs.append(song.id)
 
         if changed:
-            self.allartists = None
             self.artists.put(song.artist, artist, txn=self.txn)
 
+
     def _unindexsong_artist(self, song):
         """ delete artist information for song, but do not delete artist """
         artist = self.artists.get(song.artist, txn=self.txn)
@@ -641,12 +632,11 @@ class songdb(threading.Thread):
             # album has already been deleted
             artist.albums.remove(song.album)
 
-        self.allartists = None
         if artist.songs:
             self.artists.put(song.artist, artist, txn=self.txn)
         else:
             self.artists.delete(song.artist, txn=self.txn)
-            self.allartists = None
+            hub.notify(events.artistaddedordeleted(self.id, artist))
             
     # other indices: genre, year, rating, ...
 
@@ -828,8 +818,8 @@ class songdb(threading.Thread):
             self._indexsong_artist(newsong)
             for index in self.indices:
                 self._indexsong_index(newsong, index)
-        if (oldsong.lastplayed!=newsong.lastplayed or
-            oldsong.nrplayed!=newsong.nrplayed):
+        if (oldsong.lastplayed != newsong.lastplayed or
+            oldsong.nrplayed != newsong.nrplayed):
             self._indexsong_stats(newsong)
 
     def _unindexsong(self, song):
@@ -867,7 +857,6 @@ class songdb(threading.Thread):
             self._txn_begin()
             try:
                 self.songs.put(song.id, song, txn=self.txn)
-                self.allsongs = None
                 # insert into indices
                 self._indexsong(song)
             except:
@@ -889,38 +878,12 @@ class songdb(threading.Thread):
         try:
             self._unindexsong(song)
             self.songs.delete(song.id, txn=self.txn)
-            if self.allsongs:
-                del self.allsongs[self.allsongs.index(song)]
         except:
             self._txn_abort()
-            # trash cache
-            self.allsongs = None
             raise
         else:
             self._txn_commit()
 
-    def _registerplaylist(self, path):
-        playlist = dbitem.playlist(path)
-
-        # also try to register songs in playlist
-        # delete song, if this fails
-        for i in range(len(playlist.songs)):
-            try:
-                self._queryregistersong(playlist.songs[i])
-            except (IOError, OSError):
-                del playlist.songs[i]
-
-        # a resulting, non-empty playlist can be written in the database
-        if playlist.songs:
-            self._txn_begin()
-            try:
-                self.playlists.put(path, playlist, txn=self.txn)
-            except:
-                self._txn_abort()
-                raise
-            else:
-                self._txn_commit()
-
     def _updatesong(self, song):
         """updates entry of given song"""
 
@@ -931,21 +894,10 @@ class songdb(threading.Thread):
         self._txn_begin()
         try:
             oldsong = self.songs.get(song.id, txn=self.txn)
-            # update song cache if existent
-            try:
-                # Note that we rely on each song being uniquely specified
-                # by its path and the corresponding __cmp__ method of
-                # dbitem.song!
-                self.allsongs[self.allsongs.index(oldsong)] = song
-            except (ValueError, AttributeError):
-                pass
-            
             self.songs.put(song.id, song, txn=self.txn)
             self._reindexsong(oldsong, song)
         except:
             self._txn_abort()
-            # trash cache
-            self.allsongs = None
             raise
         else:
             self._txn_commit()
@@ -969,7 +921,6 @@ class songdb(threading.Thread):
             self._txn_begin()
             try:
                 self.songs.put(song.id, song, txn=self.txn)
-                self.allsongs = None
                 # insert into indices
                 self._indexsong(song)
             except:
@@ -987,7 +938,49 @@ class songdb(threading.Thread):
             self._updatesong(song)
         except IOError:
             self._delsong(song)
+
+    def _registerplaylist(self, playlist):
+        # also try to register songs in playlist and delete song, if
+        # this fails
+        paths = []
+        for path in playlist.songs:
+            try:
+                if self._queryregistersong(path) is not None:
+                    paths.append(path)
+            except (IOError, OSError):
+                pass
+        playlist.songs = paths
+
+        # a resulting, non-empty playlist can be written in the database
+        if playlist.songs:
+            self._txn_begin()
+            try:
+                self.playlists.put(playlist.path, playlist, txn=self.txn)
+                hub.notify(events.dbplaylistchanged(self.id, playlist))
+            except:
+                self._txn_abort()
+                raise
+            else:
+                self._txn_commit()
+
+    def _delplaylist(self, playlist):
+        """delete playlist from database"""
+        if not self.playlists.has_key(playlist.id):
+            raise KeyError
         
+        log.debug("delete playlist: %s" % str(playlist))
+        self._txn_begin()
+        try:
+            self.playlists.delete(playlist.id, txn=self.txn)
+            hub.notify(events.dbplaylistchanged(self.id, playlist))
+        except:
+            self._txn_abort()
+            raise
+        else:
+            self._txn_commit()
+
+    _updateplaylist = _registerplaylist
+
     def _updatealbum(self, album):
         """updates entry of given album of artist"""
         # XXX: changes of other indices not handled correctly
@@ -1007,19 +1000,9 @@ class songdb(threading.Thread):
         self._txn_begin()
         try:
             # update artist cache if existent
-            try:
-                # Note that we rely on each artist being uniquely specified
-                # by its name and the corresponding __cmp__ method of
-                # dbitem.artist!
-                oldartist = self.artists.get(artist.name, txn=self.txn)
-                self.allartists[self.allartists.index(oldartist)] = artist
-            except (ValueError, AttributeError):
-                pass
             self.artists.put(artist.name, artist, txn=self.txn)
         except:
             self._txn_abort()
-            # trash cache
-            self.allartists = None
             raise
         else:
             self._txn_commit()
@@ -1030,9 +1013,9 @@ class songdb(threading.Thread):
     # !!! It is not save to call any of the following methods when a transaction is active !!!
     ##########################################################################################
 
-    def _getsong(self, path):
-        """returns song entry with given path"""
-        song = self.songs.get(path)
+    def _getsong(self, id):
+        """returns song entry with given id"""
+        song = self.songs.get(id)
         return song
 
     def _getalbum(self, album):
@@ -1047,17 +1030,7 @@ class songdb(threading.Thread):
         """return all stored artists"""
         # use cached value if existent
         if indexname is None:
-            if self.allartists:
-                artists = self.allartists[:]
-                return artists
-            else:
-                log.debug("start rebuilding artist cache")
-                artists = map(self.artists.get, self.artists.keys())
-                log.debug("finished rebuilding artist cache")
-                newallartists = artists[:]
-                log.debug("saving artist cache")
-                self.allartists = newallartists
-                return artists
+            return map(self.artists.get, self.artists.keys())
         else:
             index = getattr(self, indexname+"s")
             # indexid always has to be a string
@@ -1093,16 +1066,8 @@ class songdb(threading.Thread):
 
         if artist is None and album is None and indexname is None:
             # return all songs in songdb
-            # use cached value if existent
-            if self.allsongs:
-                songs = self.allsongs[:]
-                return songs
-            else:
-                # songs = map(self.songs.get, self.songs.keys())
-                songs = self.songs.values()
-                newallsongs = songs[:]
-                self.allsongs = newallsongs
-                return songs
+            # songs = map(self.songs.get, self.songs.keys())
+            return self.songs.values()
 
         if indexname is None:
             if album is None:
@@ -1153,7 +1118,7 @@ class songdb(threading.Thread):
         return ratings
 
     def _getlastplayedsongs(self):
-        """return the last played songs together with the corresponding playing time"""
+        """return the last played songs"""
         return [(self.songs[songid], playingtime) for songid, playingtime in self.stats["lastplayed"]]
 
     def _gettopplayedsongs(self):
@@ -1193,115 +1158,91 @@ class songdb(threading.Thread):
             songs.extend(self._getsongsinplaylist(playlist.path))
         return songs
 
-    def _genrandomchoice(self, songs):
-        """ returns random selection of songs up to the maximal length
-        configured. Note that this method changes as a side-effect the
-        parameter songs"""
-
-        # choose item, avoiding duplicates. Stop after a predefined
-        # total length (in seconds). Take rating of songs/albums/artists
-        # into account
-        length = 0
-        result = []
-        for i in range(len(songs)):
-            # Simple heuristic algorithm to consider song ratings
-            # for random selection. Certainly not optimal!
-            while True:
-                nr = random.randrange(0, len(songs))
-                aitem = songs[nr]
-                rating = aitem.rating or 3
-                if aitem.lastplayed:
-                    last = max(0, int((time.time()-aitem.lastplayed[-1])/60))
-                    rating -= 2*math.exp(-last/(60.0*60*24))
-                if random.random()>=2.0**(-rating) or len(songs)==1:
-                    break
-
-            del songs[nr]
-            length += aitem.length
-            result.append(aitem)
-            if length>config.general.randominsertlength:
-                break
-        return result
-
-    def _selectrandom(self, request, songs):
-        """ wrapper function which restrict songs to a random selection
-        if specified by the caller"""
-        if request.random:
-            return self._genrandomchoice(songs)
-        else:
-            return songs
+    def isbusy(self):
+        """ check whether db is currently busy """
+        return self.txn is not None or self.channel.queue.qsize()>0
 
     # event handlers
 
-    def checkpointdatabase(self, event):
+    def checkpointdb(self, event):
         """flush memory pool, write checkpoint record to log and flush flog"""
-        if event.songdbid==self.id:
+        if event.songdbid == self.id:
             self._checkpoint()
-            hub.notify(events.sendeventin(events.checkpointdatabase(self.id), checkpointinterval))
 
     def updatesong(self, event):
-        if event.songdbid==self.id:
+        if event.songdbid == self.id:
             try:
                 self._updatesong(event.song)
             except KeyError:
                 pass
 
     def rescansong(self, event):
-        if event.songdbid==self.id:
+        if event.songdbid == self.id:
             try:
                 self._rescansong(event.song)
             except KeyError:
                 pass
 
     def delsong(self, event):
-        if event.songdbid==self.id:
+        if event.songdbid == self.id:
             try:
                 self._delsong(event.song)
             except KeyError:
                 pass
 
     def updatealbum(self, event):
-        if event.songdbid==self.id:
+        if event.songdbid == self.id:
             try:
                 self._updatealbum(event.album)
             except KeyError:
                 pass
 
     def updateartist(self, event):
-        if event.songdbid==self.id:
+        if event.songdbid == self.id:
             try:
                 self._updateartist(event.artist)
             except KeyError:
                 pass
 
     def registersongs(self, event):
-        if event.songdbid==self.id:
+        if event.songdbid == self.id:
             for song in event.songs:
                 try: self._registersong(song)
                 except (IOError, OSError): pass
 
     def registerplaylists(self, event):
-        if event.songdbid==self.id:
-            for path in event.paths:
-                try: self._registerplaylist(path)
+        if event.songdbid == self.id:
+            for playlist in event.playlists:
+                try: self._registerplaylist(playlist)
                 except (IOError, OSError): pass
 
+    def delplaylist(self, event):
+        if event.songdbid == self.id:
+            try:
+                self._delplaylist(event.playlist)
+            except KeyError:
+                pass
+
+    def updateplaylist(self, event):
+        if event.songdbid == self.id:
+            try:
+                self._updateplaylist(event.playlist)
+            except KeyError:
+                pass
+
     def clearstats(self, event):
         if event.songdbid == self.id:
             self._clearstats()
             
-    def quit(self, event):
-        self.done = True
-
     # request handlers
 
     def getdatabaseinfo(self, request):
-        if self.id!=request.songdbid:
+        if self.id != request.songdbid:
             raise hub.DenyRequest
         return ("local", self.basedir)
 
     def getnumberofsongs(self, request):
-        if self.id!=request.songdbid:
+        if self.id != request.songdbid:
             raise hub.DenyRequest
         return len(self.songs)
 
@@ -1342,10 +1283,10 @@ class songdb(threading.Thread):
         return self._queryregistersong(request.path)
 
     def getsong(self, request):
-        if self.id!=request.songdbid:
+        if self.id != request.songdbid:
             raise hub.DenyRequest
         try:
-            return self._getsong(request.path)
+            return self._getsong(request.id)
         except KeyError:
             return None
 
@@ -1354,7 +1295,7 @@ class songdb(threading.Thread):
             raise hub.DenyRequest
         if request.indexname!="decade":
             try:
-                songs = self._getsongs(request.artist, request.album, request.indexname, request.indexid)
+                return self._getsongs(request.artist, request.album, request.indexname, request.indexid)
             except (KeyError, AttributeError, TypeError):
                 return []
         else:
@@ -1366,7 +1307,7 @@ class songdb(threading.Thread):
                     songs.extend(self._getsongs(request.artist, request.album, indexname="year", indexid=year))
                 except (KeyError, AttributeError, TypeError):
                     pass
-        return self._selectrandom(request, songs)
+            return songs
 
     def getartists(self, request):
         if self.id!=request.songdbid:
@@ -1462,20 +1403,17 @@ class songdb(threading.Thread):
     def getlastplayedsongs(self, request):
         if self.id!=request.songdbid:
             raise hub.DenyRequest
-        # Due to the different structure of the result compared to the
-        # other request, namely a list of tuples (song, playingtime), we
-        # do not support random song selection here.
         return self._getlastplayedsongs()
 
     def gettopplayedsongs(self, request):
         if self.id!=request.songdbid:
             raise hub.DenyRequest
-        return self._selectrandom(request, self._gettopplayedsongs())
+        return self._gettopplayedsongs()
 
     def getlastaddedsongs(self, request):
         if self.id!=request.songdbid:
             raise hub.DenyRequest
-        return self._selectrandom(request, self._getlastaddedsongs())
+        return self._getlastaddedsongs()
 
     def getplaylist(self, request):
         if self.id!=request.songdbid:
@@ -1490,61 +1428,62 @@ class songdb(threading.Thread):
     def getsongsinplaylist(self, request):
         if self.id!=request.songdbid:
             raise hub.DenyRequest
-        return self._selectrandom(request, self._getsongsinplaylist(request.path))
+        return self._getsongsinplaylist(request.path)
 
     def getsongsinplaylists(self, request):
         if self.id!=request.songdbid:
             raise hub.DenyRequest
-        return self._selectrandom(request, self._getsongsinplaylists())
+        return self._getsongsinplaylists()
 
 #
 # thread for automatic registering and rescanning of songs in database
 #
 
-class songautoregisterer(threading.Thread):
+class songautoregisterer(service.service):
 
-    def __init__(self, basedir, songdbid, songdbchannel,
+    def __init__(self, basedir, songdbid, dbbusymethod,
                  tracknrandtitlere, tagcapitalize, tagstripleadingarticle, tagremoveaccents):
-        threading.Thread.__init__(self)
+        service.service.__init__(self, "songautoregisterer", daemonize=True)
         self.basedir = basedir
         self.songdbid = songdbid
-        self.songdbchannel = songdbchannel
+        self.dbbusymethod = dbbusymethod
         self.tracknrandtitlere = tracknrandtitlere
         self.tagcapitalize = tagcapitalize
         self.tagstripleadingarticle = tagstripleadingarticle
         self.tagremoveaccents = tagremoveaccents
         self.done = False
+        # support file extensions
+        self.supportedextensions = metadata.getextensions()
 
-        # as independent thread, we want our own event and request channel
-        self.channel = hub.newchannel()
         self.channel.subscribe(events.autoregistersongs, self.autoregistersongs)
         self.channel.subscribe(events.rescansongs, self.rescansongs)
-        self.channel.subscribe(events.quit, self.quit)
-
-        # we are only servants of higher powers
-        self.setDaemon(1)
 
     def registerdirtree(self, dir):
-        """ scan for songs and playlists in dir and its subdirectories, returning all songs which have been scanned """
+        """ scan for songs and playlists in dir and its subdirectories, returning all items which have been scanned """
         self.channel.process()
         if self.done: return []
-        paths = []
-        registeredsongs = []
+        songpaths = []
+        playlistpaths = []
+        registereditems = []
+
+        # scan for paths of songs and playlists and recursively call registering of subdirectories
         for name in os.listdir(dir):
             path = os.path.join(dir, name)
+            extension = os.path.splitext(path)[1].lower()
             if os.access(path, os.R_OK):
                 if os.path.isdir(path):
-                    try: registeredsongs.extend(self.registerdirtree(path))
-                    except (IOError, OSError): pass
-                elif name.lower().endswith(".mp3"):
-                    paths.append(path)
-                elif name.lower().endswith(".ogg") and dbitem.oggsupport:
-                    paths.append(path)
-                elif name.lower().endswith(".m3u"):
-                    hub.notify(events.registerplaylists(self.songdbid, [path]), -100)
-                    
+                    try:
+                        registereditems.extend(self.registerdirtree(path))
+                    except (IOError, OSError), e:
+                        log.warning("songautoregisterer: could not enter dir %s: %s" % (path, e))
+                elif extension in self.supportedextensions:
+                    songpaths.append(path)
+                elif extension == ".m3u":
+                    playlistpaths.append(path)
+
+        # now register songs...
         songs = []
-        for path in paths:
+        for path in songpaths:
             if self.basedir.endswith("/"):
                songid = path[len(self.basedir):]
             else:
@@ -1554,17 +1493,25 @@ class songautoregisterer(threading.Thread):
                                      self.tagcapitalize, self.tagstripleadingarticle, self.tagremoveaccents))
         if songs:
             hub.notify(events.registersongs(self.songdbid, songs), -100)
-            while self.songdbchannel.queue.qsize()>0:
+        registereditems.extend(songs)
+
+        # ... and playlists
+        playlists = [dbitem.playlist(path) for path in playlistpaths]
+        if playlists:
+            hub.notify(events.registerplaylists(self.songdbid, playlists), -100)
+
+        # do not stress the database too much
+        if songs or playlists:
+            while self.dbbusymethod():
                 time.sleep(0.1)
 
-        registeredsongs.extend(songs)
-        return registeredsongs
+        registereditems.extend(playlists)
+        return registereditems
 
     def run(self):
         # wait a little bit to not disturb the startup too much
         time.sleep(2)
-        while not self.done:
-            self.channel.process(blocking=True)
+        service.service.run(self)
 
     #
     # event handler
@@ -1580,25 +1527,39 @@ class songautoregisterer(threading.Thread):
             hub.notify(events.updatesong(self.songdbid, song))
         except IOError:
             hub.notify(events.delsong(self.songdbid, song))
-    
+
+
+    def rescanplaylist(self, playlist):
+        try:
+            newplaylist = dbitem.playlist(playlist.path)
+            hub.notify(events.updateplaylist(self.songdbid, newplaylist))
+        except IOError:
+            hub.notify(events.delplaylist(self.songdbid, playlist))
+
     def autoregistersongs(self, event):
         if self.songdbid == event.songdbid:
             log.info(_("database %s: scanning for songs in %s") % (self.songdbid, self.basedir))
 
-            # get all songs currently stored in the database
+            # get all songs and playlists currently stored in the database
             oldsongs = hub.request(requests.getsongs(self.songdbid))
+            oldplaylists = hub.request(requests.getplaylists(self.songdbid))
 
-            # scan for all songs in the filesystem
-            registeredsongs = self.registerdirtree(self.basedir)
+            # scan for all songs and playlists in the filesystem
+            registereditems = self.registerdirtree(self.basedir)
 
             # update information for songs which have not yet been scanned (in particular
-            # removing songs which are no longer present in the database)
+            # remove songs which are no longer present in the database)
             for song in oldsongs:
-                if song not in registeredsongs:
+                if song not in registereditems:
                     self.rescansong(song)
-                    while self.songdbchannel.queue.qsize()>0:
+                    while self.dbbusymethod():
                         time.sleep(0.1)
-            
+            for playlist in oldplaylists:
+                if playlist not in registereditems:
+                    self.rescanplaylist(playlist)
+                    while self.dbbusymethod():
+                        time.sleep(0.1)
+
             log.info(_("database %s: finished scanning for songs in %s") % (self.songdbid, self.basedir))
 
     def rescansongs(self, event):
@@ -1607,9 +1568,7 @@ class songautoregisterer(threading.Thread):
 
             for song in event.songs:
                 self.rescansong(song)
-                while self.songdbchannel.queue.qsize()>0:
+                while self.dbbusymethod():
                     time.sleep(0.1)
             log.info(_("database %s: finished rescanning %d songs") % (self.songdbid, len(event.songs)))
 
-    def quit(self, event):
-        self.done = True
