@@ -18,27 +18,24 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import copy
-import threading
 
 import events, hub, requests
 import log
 import network
+import service
 
 #
 # songdb class
 #
 
-class songdb(threading.Thread):
-    def __init__(self, id, server, port):
-        threading.Thread.__init__(self)
+class songdb(service.service):
+    def __init__(self, id, server, port, songdbhub):
+        service.service.__init__(self, "remote songdb")
         self.id = id
         self.server = server
         self.port = port
         self.remotesongdbid = "main"
         
-        # as independent thread, we want our own event and request channel
-        self.channel = hub.newchannel()
-        
         self.networkchannel = network.tcpclientchannel(self.server, self.port)
         #self.networkchannel.transmit(events.updatesong)
         #self.networkchannel.transmit(events.updatealbum)
@@ -52,23 +49,12 @@ class songdb(threading.Thread):
         #self.channel.subscribe(events.updateartist, self.updateartist)
         #self.channel.subscribe(events.registersongs, self.registersongs)
         #self.channel.subscribe(events.registerplaylists, self.registerplaylists)
-        self.channel.subscribe(events.quit, self.quit)
-        
-        self.done = False
         
         # we are a database service provider...
         self.channel.supply(requests.dbrequest, self.dbrequest)
         log.info(_("database %s: type remote, hostname %s, port %d") % (self.id,
                                                                          self.server, self.port))
 
-    def run(self):
-        """main loop"""
-        while not self.done:
-            self.channel.process(blocking=True)
-
-    def quit(self, event):
-        self.done = True
-
     # request handler
 
     def dbrequest(self, request):
