@@ -243,17 +243,32 @@ class window:
                 self.vline(3, xpos, curses.ACS_CKBOARD, self.ih-4, self.colors.scrollbar)
                 self.vline(scrollbarbegin, xpos, curses.ACS_CKBOARD, scrollbarheight, self.colors.scrollbarhigh)
 
+    def bottom(self):
+        """ bring window to bottom"""
+        try:
+            self.panel.bottom()
+        except:
+            pass
+        else:
+            hub.notify(events.focuschanged())
+
     def top(self):
         """ bring window to top"""
-        try: self.panel.top()
-        except: pass
-        else: hub.notify(events.focuschanged())
+        try:
+            self.panel.top()
+        except:
+            pass
+        else:
+            hub.notify(events.focuschanged())
 
     def hide(self):
         """ hide window """
-        try: self.panel.hide()
-        except: pass
-        else: hub.notify(events.focuschanged())
+        try:
+            self.panel.hide()
+        except:
+            pass
+        else:
+            hub.notify(events.focuschanged())
 
     def hasfocus(self):
         """ is window on top and has thus the current focus? """
